#include "platform.h"
#include "touchscreen.h"
#include "conf.h"
#include "font.h"
#include "lang.h"
#include "fileutil.h"
#include "gui.h"
#include "gui_lang.h"
#include "gui_draw.h"
#include "gui_menu.h"
#include "gui_mbox.h"
#include "gui_osd.h"
#include "console.h"
#include "modules.h"
#include "levent.h"
#include "callfunc.h"
#include "usb_remote.h"
#include "module_load.h"
 

// splash screen time
 
#define SPLASH_TIME 50  // Displays for 4 seconds
                        // gui_redraw called every 4th loop of core_spytask which has 20ms delay per loop = (4*20) * 20 = 1600ms

// for memory info, duplicated from lowlevel
extern const char _start,_end;
int script_run_on_alt_flag ;
static char buf[256];
int gDebug,gDebug2,gDebug3;
char gDebugString[32];
//-------------------------------------------------------------------
// 						Menu enumeration definitions 
//-------------------------------------------------------------------

/*
common code for "enum" menu items that just take a list of string values and don't require any special setters
would be better to have another menu item type that does this by default
save memory by eliminating dupe code
*/
void gui_enum_value_change(int *value, int change, unsigned num_items) {
    *value+=change;
    if (*value<0)
        *value = num_items-1;
    else if (*value>=num_items)
        *value = 0;
}

static const char* gui_change_simple_enum(int* value, int change, const char** items, unsigned num_items) {
    gui_enum_value_change(value, change, num_items);
    return (const char*)lang_str((int)items[*value]);
}

const char* gui_change_enum2(const CMenuItem *menu_item, int change)
{
    const char** items = (const char**)menu_item->arg;
    gui_enum_value_change(menu_item->value, change, menu_item->opt_len);
    return (const char*)lang_str((int)items[*menu_item->value]);
}

//-------------------------------------------------------------------
//							USB, A/D and Remote switch
//-------------------------------------------------------------------

static const char* gui_USB_switch_types_enum(int change, int arg)
{
    static const char* modes[] = { "None","OnePush", "TwoPush", "CA-1" };    // note : make sure # of entries less than NUM_USB_INPUT_DRV in usb_remote.c
    gui_enum_value_change(&conf.remote_switch_type,change,sizeof(modes)/sizeof(modes[0]));

    if (change) set_usb_remote_state();

    return modes[conf.remote_switch_type];
}

static const char* gui_USB_control_modes_enum(int change, int arg)
{
    static const char* modes[] = { "None", "Normal", "Quick", "Burst", "Bracket","Zoom", "Video" }; // note : make sure # of entries less than NUM_USB_MODULES in usb_remote.c
    gui_enum_value_change(&conf.remote_control_mode,change,sizeof(modes)/sizeof(modes[0]));

    if (change) set_usb_remote_state();

    return modes[conf.remote_control_mode];
}

#ifdef CAM_REMOTE_AtoD_CHANNEL
static const char* gui_remote_channels[] = { "USB", "A/D Ch" };
#endif

#ifndef CAM_REMOTE_USES_PRECISION_SYNC
static CMenuItem synch_delay[2] = {
    MENU_ITEM   (0, 0,  MENUITEM_INT|MENUITEM_F_UNSIGNED,   &conf.synch_delay_value,        0 ),
    MENU_ITEM   (0, 0,  MENUITEM_BOOL,                      &conf.synch_delay_enable,       0 ),
};
#endif

static CMenuItem remote_submenu_items[] = {
    MENU_ITEM   (0,LANG_MENU_REMOTE_ENABLE,              MENUITEM_BOOL|MENUITEM_ARG_CALLBACK, &conf.remote_enable, (int)set_usb_remote_state),
#ifdef CAM_REMOTE_AtoD_CHANNEL
    MENU_ENUM2  (0,LANG_MENU_REMOTE_INPUT_CHANNEL,                                   &conf.remote_input_channel,   gui_remote_channels ),
#endif
    MENU_ITEM   (0,LANG_MENU_REMOTE_DEVICE,              MENUITEM_ENUM,                gui_USB_switch_types_enum, 0),
    MENU_ITEM   (0,LANG_MENU_REMOTE_LOGIC,  	            MENUITEM_ENUM,                gui_USB_control_modes_enum, 0),
    MENU_ITEM   (0,LANG_MENU_SYNCH_ENABLE,               MENUITEM_BOOL,                &conf.synch_enable, 0),
    MENU_ITEM   (0,LANG_MENU_SCRIPT_START_ENABLE,        MENUITEM_BOOL,                &conf.remote_enable_scripts, 0),   
    MENU_ITEM   (0,LANG_MENU_BACK,                       MENUITEM_UP,                  0, 0),
    {0}
};
static CMenu remote_submenu = {0,LANG_MENU_REMOTE_PARAM_TITLE, remote_submenu_items };

//-------------------------------------------------------------------
//                                AUTO ISO
//-------------------------------------------------------------------

static const char* gui_autoiso_shutter_modes[] = { "Auto", "1/2", "1/4", "1/6", "1/8", "1/15", "1/30", "1/60", "1/125", "1/250", "1/500", "1/1000", "1/2000" };

static const char* gui_autoiso2_shutter_modes[] = { "Off", "1", "1/2","1/4", "1/6", "1/8", "1/12", "1/15", "1/20", "1/25", "1/30",
                 	                                "1/40", "1/50", "1/60", "1/80", "1/100", "1/125", "1/160", "1/250", "1/500", "1/1000", "1/2000" };

static const char* gui_overexp_ev_modes[] = { "Off", "-1/3 Ev", "-2/3 Ev", "-1 Ev", "-1 1/3Ev", "-1 2/3Ev", "-2 Ev" };

static CMenuItem autoiso_submenu_items[] = {
    MENU_ITEM   (0,LANG_MENU_AUTOISO_ENABLED,            MENUITEM_BOOL,                                      &conf.autoiso_enable,           0 ),
    MENU_ENUM2	(0,LANG_MENU_AUTOISO_MIN_SHUTTER,        &conf.autoiso_shutter_enum,                         gui_autoiso_shutter_modes ),
    MENU_ITEM   (0,LANG_MENU_AUTOISO_USER_FACTOR,        MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.autoiso_user_factor,      MENU_MINMAX(1, 8) ),

#if CAM_HAS_IS
    MENU_ITEM   (0,LANG_MENU_AUTOISO_IS_FACTOR,          MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.autoiso_is_factor,        MENU_MINMAX(1, 8) ),
#endif

    MENU_ITEM   (0,LANG_MENU_AUTOISO_MIN_ISO,            MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.autoiso_min_iso,          MENU_MINMAX(10, 200) ),
    MENU_ITEM	(0,LANG_MENU_AUTOISO_MAX_ISO_AUTO,       MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.autoiso_max_iso_auto,     MENU_MINMAX(10, 3200) ),

#if CAM_HAS_HI_ISO_AUTO_MODE
    MENU_ITEM   (0,LANG_MENU_AUTOISO_MAX_ISO_HI,         MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.autoiso_max_iso_hi,       MENU_MINMAX(200, 3200) ),
#endif

    MENU_ENUM2  (0,LANG_MENU_AUTOISO_MIN_SHUTTER2,       &conf.autoiso2_shutter_enum,                        gui_autoiso2_shutter_modes ), 
    MENU_ITEM	(0,LANG_MENU_AUTOISO_MAX_ISO2,           MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.autoiso2_max_iso_auto,	MENU_MINMAX(100, 3200) ),

    MENU_ENUM2	(0,LANG_MENU_AUTOISO_OVEREXP_EV,        &conf.overexp_ev_enum, gui_overexp_ev_modes ), 
    MENU_ITEM	(0,LANG_MENU_AUTOISO_OVEREXP_THRES,     MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.overexp_threshold, 	MENU_MINMAX(1, 20) ),

    MENU_ITEM   (0,LANG_MENU_BACK,                       MENUITEM_UP,    0,                                                              0 ),
    {0}
};

static CMenu autoiso_submenu = {0x2d,LANG_MENU_AUTOISO_TITLE, autoiso_submenu_items };

//-------------------------------------------------------------------
// 					Save camera romlog to A/ROMLOG.LOG file
//-------------------------------------------------------------------

static void save_romlog(int arg)
{
    extern unsigned _ExecuteEventProcedure(const char *name,...);

    if (stat("A/ROMLOG.LOG",0)    == 0) remove("A/ROMLOG.LOG");
    if (stat("A/RomLogErr.txt",0) == 0) remove("A/RomLogErr.txt");

    unsigned args[3];
    args[0] = (unsigned)"SystemEventInit";
    if (call_func_ptr(_ExecuteEventProcedure,args,1) == -1)
    {
        args[0] = (unsigned)"System.Create";
        if (call_func_ptr(_ExecuteEventProcedure,args,1) == -1)
        {
            gui_mbox_init(LANG_ERROR, LANG_SAVE_ROMLOG_INIT_ERROR, MBOX_BTN_OK|MBOX_TEXT_CENTER, NULL);
            return;
        }
    }

    args[0] = (unsigned)"GetLogToFile";
    args[1] = (unsigned)"A/ROMLOG.LOG";
    args[2] = 1;
    if (call_func_ptr(_ExecuteEventProcedure,args,3) == -1)
    {
        gui_mbox_init(LANG_ERROR, LANG_SAVE_ROMLOG_FAIL, MBOX_BTN_OK|MBOX_TEXT_CENTER, NULL);
    }
    else
    {
        gui_mbox_init(LANG_INFORMATION, LANG_SAVE_ROMLOG_OK, MBOX_BTN_OK|MBOX_TEXT_CENTER, NULL);
    }
}
 

//-------------------------------------------------------------------
//					Draw selected file and code pages
//-------------------------------------------------------------------
 
static void gui_draw_read_selected(const char *fn)
{
    if (fn)
    {
		libtxtread->read_file(fn);
    }
}

static void gui_draw_read(int arg)
{
    libfselect->file_select(LANG_STR_SELECT_TEXT_FILE, conf.reader_file, "A/CHDK/BOOKS", gui_draw_read_selected);
}

static void gui_draw_read_last(int arg)
{
    if (stat(conf.reader_file,0) == 0)
        gui_draw_read_selected(conf.reader_file);
    else
        gui_draw_read(arg);
}
 

static const char* gui_reader_codepage_cps[] = { "Win1251", "DOS"};
static CMenuItem reader_submenu_items[] = {
    MENU_ITEM(0,LANG_MENU_READ_OPEN_NEW,           MENUITEM_PROC,    gui_draw_read, 0 ),
    MENU_ITEM(0,LANG_MENU_READ_OPEN_LAST,          MENUITEM_PROC,    gui_draw_read_last, 0 ),
    MENU_ENUM2(0,LANG_MENU_READ_CODEPAGE,          &conf.reader_codepage, gui_reader_codepage_cps ),
    MENU_ITEM(0,LANG_MENU_READ_WORD_WRAP,          MENUITEM_BOOL,    &conf.reader_wrap_by_words, 0 ),
    MENU_ITEM(0,LANG_MENU_READ_AUTOSCROLL,         MENUITEM_BOOL,    &conf.reader_autoscroll, 0 ),
    MENU_ITEM(0,LANG_MENU_READ_AUTOSCROLL_DELAY,   MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.reader_autoscroll_delay, MENU_MINMAX(0, 60) ),
    MENU_ITEM(0,LANG_MENU_BACK,                    MENUITEM_UP, 0, 0 ),
    {0}
};

static CMenu reader_submenu = {0,LANG_MENU_READ_TITLE, reader_submenu_items };

//-------------------------------------------------------------------
//								Tools and Modules
//-------------------------------------------------------------------

static CMenu tools_submenu = {0,LANG_MENU_MISC_TOOLS, 0 };

int submenu_sort(const void* v1, const void* v2)
{
    CMenuItem *mi1 = (CMenuItem*)v1;
    CMenuItem *mi2 = (CMenuItem*)v2;

    return strcmp(lang_str(mi1->text), lang_str(mi2->text));
}

static CMenuItem* create_module_menu(int mtype, char symbol)
{
    DIR             *d;
    struct dirent   *de;
    int             mcnt = 0;
    ModuleInfo      mi;
    char            modName[33];
    char            *nm;

    // Open directory & count # of modules matching mtype
    d = opendir("A/CHDK/MODULES");

    if (d)
    {
        while ((de = readdir(d)))
        {
            if ((de->d_name[0] != 0xE5) && (strcmp(de->d_name,".") != 0) && (strcmp(de->d_name,"..") != 0))
            {
                get_module_info(de->d_name, &mi, 0, 0);
                if ((mi.moduleType & MTYPE_MASK) == mtype)
                    mcnt++;
            }
        }

        closedir(d);
    }

    // Allocate memory for menu
    CMenuItem *submenu = malloc((mcnt+2) * sizeof(CMenuItem));
    memset(submenu, 0, (mcnt+2) * sizeof(CMenuItem));

    // Re-open directory & create game menu
    d = opendir("A/CHDK/MODULES");

    if (d)
    {
        mcnt = 0;
        while ((de = readdir(d)))
        {
            if ((de->d_name[0] != 0xE5) && (strcmp(de->d_name,".") != 0) && (strcmp(de->d_name,"..") != 0))
            {
                get_module_info(de->d_name, &mi, modName, 33);
                if ((mi.moduleType & MTYPE_MASK) == mtype)
                {
                    submenu[mcnt].char_size = 0;
                    if (mi.moduleType & MTYPE_SUBMENU_TOOL)
                        submenu[mcnt].type = MENUITEM_SUBMENU_PROC;
                    else
                        submenu[mcnt].type = MENUITEM_PROC;
                    if (mi.moduleName < 0)
                        submenu[mcnt].text = -mi.moduleName;    // LANG string
                    else
                    {
                        nm = malloc(strlen(modName)+1);
                        strcpy(nm, modName);
                        submenu[mcnt].text = (int)nm;
                    }
                    submenu[mcnt].value = (int*)module_run;
                    nm = malloc(strlen(de->d_name)+1);
                    strcpy(nm, de->d_name);
                    submenu[mcnt].arg = (int)nm;
                    mcnt++;
                }
            }
        }

        closedir(d);

        submenu[mcnt].char_size = 0;
        submenu[mcnt].type = MENUITEM_UP;
        submenu[mcnt].text = LANG_MENU_BACK;
    }

    if (mcnt > 0)
    {
        extern int submenu_sort_arm(const void* v1, const void* v2);
        qsort(submenu, mcnt, sizeof(CMenuItem), submenu_sort_arm);
    }

    return submenu;
}

static void gui_module_menu(CMenu *m, int type)
{
    if (m->menu == 0)
        m->menu = create_module_menu(type, m->title_char_size);
    gui_activate_sub_menu(m);
}
 
static void gui_tools_menu(int arg)
{
    gui_module_menu(&tools_submenu, MTYPE_TOOL);
}

//-------------------------------------------------------------------
//								SD card support
//-------------------------------------------------------------------

static void gui_menuproc_mkbootdisk(int arg)
{
    mark_filesystem_bootable();
    gui_mbox_init(LANG_INFORMATION, LANG_CONSOLE_TEXT_FINISHED, MBOX_BTN_OK|MBOX_TEXT_CENTER|MBOX_FUNC_RESTORE, NULL);
}

#if CAM_MULTIPART

static void card_break_proc(unsigned int btn)
{
    if (btn==MBOX_BTN_YES) create_partitions();
}

static void gui_menuproc_break_card(int arg)
{
    gui_mbox_init(LANG_WARNING, LANG_PARTITIONS_CREATE_WARNING, MBOX_BTN_YES_NO|MBOX_DEF_BTN2|MBOX_TEXT_CENTER|MBOX_FUNC_RESTORE, card_break_proc);
}

static char* partitions_enum=NULL;

static const char* gui_menuproc_swap_partitions_enum(int change, int arg)
{
    int new_partition;
    int partition_count = get_part_count();
    char vBuf[16];
    if(partitions_enum)
    {
      free(partitions_enum);
      partitions_enum=NULL;
    }
    new_partition= get_active_partition()+change;
    if( new_partition <=0)
    {
      new_partition = partition_count;
    }
    else if( new_partition > partition_count)
    {
      new_partition = 1;
    }  
    sprintf(vBuf,"%d/%d",new_partition, partition_count);
    partitions_enum=malloc((strlen(vBuf)+1)*sizeof(char));
    strcpy(partitions_enum,vBuf);

    if(change != 0)
    {
      swap_partitions(new_partition);
    }
    return partitions_enum;
}

#endif

static CMenuItem sdcard_submenu_items[] = {
    MENU_ITEM   (0,LANG_MENU_DEBUG_MAKE_BOOTABLE,        MENUITEM_PROC,                  gui_menuproc_mkbootdisk, 0 ),
#if CAM_MULTIPART
    MENU_ITEM   (0,LANG_MENU_DEBUG_CREATE_MULTIPART ,    MENUITEM_PROC,                  gui_menuproc_break_card,            0 ),
    MENU_ITEM   (0,LANG_MENU_DEBUG_SWAP_PART,            MENUITEM_ENUM,                  gui_menuproc_swap_partitions_enum,  0 ),
#endif
    MENU_ITEM   (0,LANG_MENU_BACK,                       MENUITEM_UP,                    0,                                  0 ),
    {0},
};

static CMenu sdcard_submenu = {0,LANG_SD_CARD, sdcard_submenu_items };
 

//-------------------------------------------------------------------
//			Display Build,memory,Lua native calls and unsafe I/o
//-------------------------------------------------------------------

static void gui_draw_fselect(int arg)
{
    libfselect->file_select(LANG_STR_FILE_BROWSER, "A", "A", NULL);
}

static void gui_show_build_info(int arg)
{
    static char comp[64];

#ifdef __GNUC__
# ifndef __GNUC_PATCHLEVEL__
# define __GNUC_PATCHLEVEL 0
# endif
    sprintf(comp, "GCC %d.%d.%d", __GNUC__ ,__GNUC_MINOR__,__GNUC_PATCHLEVEL__ );
#else
    sprintf(comp, "UNKNOWN" );
#endif
    sprintf(buf,lang_str(LANG_MSG_BUILD_INFO_TEXT), camera_info.chdk_ver, camera_info.build_number, camera_info.build_svnrev, camera_info.build_date, camera_info.build_time, camera_info.platform, camera_info.platformsub, comp);
    gui_mbox_init(LANG_MSG_BUILD_INFO_TITLE, (int)buf, MBOX_FUNC_RESTORE|MBOX_TEXT_LEFT, NULL);
}

static void gui_show_memory_info(int arg)
{
    sprintf(buf, lang_str(LANG_MSG_MEMORY_INFO_TEXT), core_get_free_memory(), camera_info.memisosize, &_start, &_end);
    gui_mbox_init(LANG_MSG_MEMORY_INFO_TITLE, (int)buf, MBOX_FUNC_RESTORE|MBOX_TEXT_CENTER, NULL);
}

#if !defined(OPT_FORCE_LUA_CALL_NATIVE)
static void lua_native_call_warning(unsigned int btn)
{
    if (btn==MBOX_BTN_NO)
        conf.script_allow_lua_native_calls = 0;
}

static void gui_lua_native_call_warning()
{
    if (conf.script_allow_lua_native_calls)
        gui_mbox_init(LANG_WARNING, LANG_MENU_LUA_NATIVE_CALLS_WARNING, MBOX_BTN_YES_NO|MBOX_DEF_BTN2|MBOX_TEXT_CENTER, lua_native_call_warning);
}
#endif

#if defined(CAM_IS_VID_REC_WORKS)
static void unsafe_io_warning(unsigned int btn)
{
    if (btn==MBOX_BTN_NO)
        conf.allow_unsafe_io = 0;
}

static void gui_unsafe_io_warning()
{
    if (conf.allow_unsafe_io)
        gui_mbox_init(LANG_WARNING, LANG_MENU_ALLOW_UNSAFE_IO_WARNING, MBOX_BTN_YES_NO|MBOX_DEF_BTN2|MBOX_TEXT_CENTER, unsafe_io_warning);
}
#endif

//-------------------------------------------------------------------
//								Console
//-------------------------------------------------------------------
static const char* gui_console_show_enum[]={ "ALT", "Always" };

static void gui_console_clear(int arg)
{
    console_close();
    gui_mbox_init(LANG_MENU_CONSOLE_CLEAR, LANG_MENU_CONSOLE_RESET, MBOX_BTN_OK|MBOX_TEXT_CENTER, NULL);
}

static void gui_console_show(int arg)
{
    void display_console();
    display_console();
}

static CMenuItem console_settings_submenu_items[] = {
    MENU_ENUM2(0,LANG_MENU_CONSOLE_SHOWIN,       &conf.console_show,         gui_console_show_enum ),
    MENU_ITEM(0,LANG_MENU_CONSOLE_TIMEOUT,       MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.console_timeout, MENU_MINMAX(3, 30) ),
    MENU_ITEM(0,LANG_MENU_CONSOLE_SHOW,          MENUITEM_PROC,              gui_console_show, 0 ),
    MENU_ITEM(0,LANG_MENU_CONSOLE_CLEAR,         MENUITEM_PROC,              gui_console_clear, 0 ),
    MENU_ITEM(0,LANG_MENU_BACK,                  MENUITEM_UP, 0, 0 ),
    {0}
};

static CMenu console_settings_submenu = {0,LANG_MENU_CONSOLE_SETTINGS, console_settings_submenu_items };

//-------------------------------------------------------------------------------------------
// Misc - File browser,text reader,console,flashlight,Build,SD,Debug,Lua native,unsafe I/O
//-------------------------------------------------------------------------------------------

static CMenuItem misc_submenu_items[] = {
    MENU_ITEM   (0,LANG_MENU_MISC_FILE_BROWSER,          MENUITEM_PROC,                  gui_draw_fselect,                   0 ),    
    MENU_ITEM   (0,LANG_MENU_MISC_TEXT_READER,           MENUITEM_SUBMENU,               &reader_submenu,                    0 ), 
    MENU_ITEM   (0,LANG_MENU_CONSOLE_SETTINGS,           MENUITEM_SUBMENU,               &console_settings_submenu, 0 ),
#if CAM_SWIVEL_SCREEN
    MENU_ITEM   (0,LANG_MENU_MISC_FLASHLIGHT,            MENUITEM_BOOL,                  &conf.flashlight, 0 ),
#endif
    MENU_ITEM   (0,LANG_MENU_MISC_BUILD_INFO,            MENUITEM_PROC,                  gui_show_build_info, 0 ),
    MENU_ITEM   (0,LANG_MENU_MISC_MEMORY_INFO,           MENUITEM_PROC,                  gui_show_memory_info, 0 ),
#if !defined(OPT_FORCE_LUA_CALL_NATIVE)
    MENU_ITEM   (0,LANG_MENU_ENABLE_LUA_NATIVE_CALLS,    MENUITEM_BOOL|MENUITEM_ARG_CALLBACK, &conf.script_allow_lua_native_calls, (int)gui_lua_native_call_warning ),
#endif
#if defined(CAM_IS_VID_REC_WORKS)
    MENU_ITEM   (0,LANG_MENU_ENABLE_UNSAFE_IO,           MENUITEM_BOOL|MENUITEM_ARG_CALLBACK, &conf.allow_unsafe_io, (int)gui_unsafe_io_warning ),
#endif
#if defined(CAM_DRYOS)
    MENU_ITEM   (0,LANG_MENU_DISABLE_LFN_SUPPORT,        MENUITEM_BOOL,                  &conf.disable_lfn_parser_ui, 0 ),
#endif
    MENU_ITEM   (0,LANG_SD_CARD,                         MENUITEM_SUBMENU,               &sdcard_submenu,                    0 ),
#ifdef OPT_DEBUGGING
    MENU_ITEM   (0,LANG_MENU_MAIN_DEBUG,                 MENUITEM_SUBMENU,               &debug_submenu,                     0 ),
#endif
    MENU_ITEM   (0,LANG_MENU_BACK,                       MENUITEM_UP,                    0,                                  0 ),
    {0},
};

static CMenu misc_submenu = {0,LANG_MENU_MISC_TITLE, misc_submenu_items };

//-------------------------------------------------------------------
//								Battery
//-------------------------------------------------------------------

static void cb_perc()
{
    conf.batt_volts_show=0;
}

static void cb_volts()
{
    conf.batt_perc_show=0;
}

static void cb_batt_max()
{
    if (conf.batt_volts_max < conf.batt_volts_min + 25)
        conf.batt_volts_min = conf.batt_volts_max - 25;
}

static void cb_batt_min()
{
    if (conf.batt_volts_min > conf.batt_volts_max - 25)
        conf.batt_volts_max = conf.batt_volts_min + 25;
}

static CMenuItem battery_submenu_items[] = {
    MENU_ITEM   (0,LANG_MENU_BATT_VOLT_MAX,              MENUITEM_INT|MENUITEM_ARG_CALLBACK,     &conf.batt_volts_max,   (int)cb_batt_max ),
    MENU_ITEM   (0,LANG_MENU_BATT_VOLT_MIN,              MENUITEM_INT|MENUITEM_ARG_CALLBACK,     &conf.batt_volts_min,   (int)cb_batt_min ),
    MENU_ITEM   (0,LANG_MENU_BATT_SHOW_PERCENT,          MENUITEM_BOOL|MENUITEM_ARG_CALLBACK,    &conf.batt_perc_show,   (int)cb_perc ),
    MENU_ITEM   (0,LANG_MENU_BATT_SHOW_VOLTS,            MENUITEM_BOOL|MENUITEM_ARG_CALLBACK,    &conf.batt_volts_show,  (int)cb_volts ),
    MENU_ITEM   (0,LANG_MENU_BATT_SHOW_ICON,             MENUITEM_BOOL,                          &conf.batt_icon_show,   0 ),
    MENU_ITEM   (0,LANG_MENU_BACK,                       MENUITEM_UP,                            0,                      0 ),
    {0}
};

static CMenu battery_submenu = {0,LANG_MENU_BATT_TITLE, battery_submenu_items };

//-------------------------------------------------------------------
//								Grid lines
//-------------------------------------------------------------------
 
static void gui_grid_lines_load_selected(const char *fn)
{
    if (fn)
        libgrids->grid_lines_load(fn);
}

static void gui_grid_lines_load(int arg)
{
    libfselect->file_select(LANG_STR_SELECT_GRID_FILE, conf.grid_lines_file, "A/CHDK/GRIDS", gui_grid_lines_load_selected);
}

static CMenuItem grid_submenu_items[] = {
    MENU_ITEM(0,LANG_MENU_SHOW_GRID,         MENUITEM_BOOL,		&conf.show_grid_lines, 0 ),
    MENU_ITEM(0,LANG_MENU_GRID_LOAD,         MENUITEM_PROC,		gui_grid_lines_load, 0 ),
    MENU_ITEM(0,(int)conf.grid_title,         MENUITEM_TEXT,      0, 0 ),
    MENU_ITEM(0,LANG_MENU_BACK,              MENUITEM_UP, 0, 0 ),
    {0}
};

static CMenu grid_submenu = {0,LANG_MENU_GRID_TITLE, grid_submenu_items };
// --------------------------------------------------------------------------------
 
#ifdef  CAM_TOUCHSCREEN_UI

static const char* gui_touchscreen_disable_modes[]=         { "Enable", "Disable" };

static CMenuItem touchscreen_submenu_items[] = {
    MENU_ENUM2  (0,LANG_MENU_TS_VIDEO_AE_DISABLE,        &conf.touchscreen_disable_video_controls,    gui_touchscreen_disable_modes ),
    MENU_ENUM2  (0,LANG_MENU_TS_ALT_SHORTCUTS_DISABLE,   &conf.touchscreen_disable_shortcut_controls, gui_touchscreen_disable_modes ),
    MENU_ITEM   (0,LANG_MENU_BACK,                       MENUITEM_UP,        0, 0 ),
    {0}
};

static CMenu touchscreen_submenu = {0,LANG_MENU_TOUCHSCREEN_VALUES, touchscreen_submenu_items };

#endif

//-------------------------------------------------------------------
//								   OSD
//-------------------------------------------------------------------

static void cb_change_rotate_osd()
{
    update_draw_proc();
    gui_menu_erase_and_redraw();
}

static const char* gui_hide_osd_modes[] =                   { "Don't", "In Playback", "On Disp Press", "Both" };
static const char* gui_show_usb_info_modes[] =              { "Off", "Icon", "Text" };

static CMenuItem osd_submenu_items[] = {
    MENU_ITEM(0,LANG_MENU_OSD_SHOW,              MENUITEM_BOOL,          &conf.show_osd, 0 ),
    MENU_ENUM2(0,LANG_MENU_OSD_HIDE_PLAYBACK,                            &conf.hide_osd, gui_hide_osd_modes ),
    MENU_ITEM(0,LANG_MENU_OSD_ROTATE,            MENUITEM_BOOL | MENUITEM_ARG_CALLBACK, &conf.rotate_osd, (int)cb_change_rotate_osd ),
    MENU_ITEM(0,LANG_MENU_OSD_SHOW_STATES,       MENUITEM_BOOL,          &conf.show_state, 0 ),
    MENU_ENUM2(0,LANG_MENU_USB_SHOW_INFO,                                &conf.usb_info_enable, gui_show_usb_info_modes ),
    MENU_ITEM(0,LANG_MENU_OSD_BATT_PARAMS,       MENUITEM_SUBMENU,       &battery_submenu, 0 ),
    MENU_ITEM(0,LANG_MENU_OSD_SHOW_IN_REVIEW,    MENUITEM_BOOL,          &conf.show_osd_in_review, 0 ),
    MENU_ITEM(0,LANG_MENU_OSD_SHOW_HIDDENFILES,  MENUITEM_BOOL,          &conf.show_hiddenfiles, 0 ),
#ifdef  CAM_TOUCHSCREEN_UI
    MENU_ITEM   (0,LANG_MENU_TOUCHSCREEN_VALUES,         MENUITEM_SUBMENU,   &touchscreen_submenu,       0 ),
#endif
    MENU_ITEM   (0,LANG_MENU_BACK,                       MENUITEM_UP, 0,                                 0 ),
    {0}
};

static CMenu osd_submenu = {0,LANG_MENU_OSD_TITLE, osd_submenu_items };

//-------------------------------------------------------------------
//								Languages
//-------------------------------------------------------------------

 
static void gui_draw_lang_selected(const char *fn)
{
    if (fn) {
        strcpy(conf.lang_file, fn);
        lang_load_from_file(conf.lang_file);
        gui_menu_init(NULL);
    }
}

static void gui_draw_load_lang(int arg)
{
    libfselect->file_select(LANG_STR_SELECT_LANG_FILE, conf.lang_file, "A/CHDK/LANG", gui_draw_lang_selected);
}

static const char* gui_font_enum(int change, int arg)
{
    extern int num_codepages;
    extern char* codepage_names[];

    gui_enum_value_change(&conf.font_cp,change,num_codepages);
  return codepage_names[conf.font_cp];
}

//-------------------------------------------------------------------
//								Menus
//-------------------------------------------------------------------
 static void gui_menuproc_reset_files(int arg)
{
    conf.lang_file[0] = 0;
    conf_save();
    gui_mbox_init(LANG_INFORMATION, LANG_MENU_RESTART_CAMERA, MBOX_BTN_OK|MBOX_TEXT_CENTER, NULL);
}

static const char* gui_text_box_charmap[] = { "Default", "German", "Russian" };

static CMenuItem menu_font_submenu_items[] = {
    MENU_ITEM(0,LANG_MENU_VIS_LANG,                MENUITEM_PROC,      gui_draw_load_lang, 0 ),
    MENU_ITEM(0,LANG_MENU_VIS_OSD_FONT,            MENUITEM_ENUM,      gui_font_enum, &conf.font_cp ),
    MENU_ENUM2(0,LANG_MENU_VIS_CHARMAP,            &conf.tbox_char_map, gui_text_box_charmap ),
    MENU_ITEM(0,LANG_MENU_RESET_FILES,             MENUITEM_PROC, 	  gui_menuproc_reset_files, 0 ),
    MENU_ITEM(0,LANG_MENU_BACK,                    MENUITEM_UP, 0, 0 ),
    {0}
};

static CMenu menu_font_submenu = {0,LANG_MENU_FONT_SETTINGS, menu_font_submenu_items };

static CMenuItem menu_settings_submenu_items[] = {
    MENU_ITEM(0,LANG_MENU_VIS_MENU_CENTER,       MENUITEM_BOOL,	        &conf.menu_center, 0 ),
    MENU_ITEM(0,LANG_MENU_SELECT_FIRST_ENTRY,    MENUITEM_BOOL,	        &conf.menu_select_first_entry, 0 ),
    MENU_ITEM(0,LANG_MENU_SHOW_ALT_HELP,         MENUITEM_BOOL,          &conf.show_alt_helper, 0 ),
    MENU_ITEM(0,LANG_MENU_SHOW_ALT_HELP_DELAY,   MENUITEM_INT|MENUITEM_F_UNSIGNED|MENUITEM_F_MINMAX, &conf.show_alt_helper_delay, MENU_MINMAX(0, 10) ),
    MENU_ITEM(0,LANG_MENU_FONT_SETTINGS,         MENUITEM_SUBMENU,       &menu_font_submenu, 0 ),
    MENU_ITEM(0,LANG_MENU_BACK,                  MENUITEM_UP, 0, 0 ),
    {0}
};

static CMenu menu_settings_submenu = {0,LANG_MENU_MENU_SETTINGS, menu_settings_submenu_items };

//-------------------------------------------------------------------
//						   ALT button and extra buttons
//-------------------------------------------------------------------

#if CAM_ADJUSTABLE_ALT_BUTTON

const char* gui_alt_mode_button_enum(int change, int arg)
{
#if defined(CAM_ALT_BUTTON_NAMES) && defined(CAM_ALT_BUTTON_OPTIONS)
    static const char* names[] = CAM_ALT_BUTTON_NAMES;
    static const int keys[] = CAM_ALT_BUTTON_OPTIONS;
#else
#error Make sure CAM_ALT_BUTTON_NAMES and CAM_ALT_BUTTON_OPTIONS are defined in platform_camera.h
#endif
    int i;

    for (i=0; i<sizeof(names)/sizeof(names[0]); ++i) {
        if (conf.alt_mode_button==keys[i]) {
            break;
        }
    }

    i+=change;
    if (i<0)
        i=(sizeof(names)/sizeof(names[0]))-1;
    else if (i>=(sizeof(names)/sizeof(names[0])))
        i=0;

    conf.alt_mode_button = keys[i];
    return names[i];
}
#endif
 
static const char* gui_alt_power_enum(int change, int arg)
{
// Script option is retained even if scripting is disabled, otherwise conf values will change
// Equivalent to ALT
    static const char* modes[]={ "Never", "Alt", "Script", "Always" };

    gui_enum_value_change(&conf.alt_prevent_shutdown,change,sizeof(modes)/sizeof(modes[0]));
        
    return modes[conf.alt_prevent_shutdown];
}

static void gui_menuproc_reset_selected(unsigned int btn)
{
    if (btn==MBOX_BTN_YES)
        conf_load_defaults();
}

static void gui_menuproc_reset(int arg)
{
    gui_mbox_init(LANG_MSG_RESET_OPTIONS_TITLE, 
                  LANG_MSG_RESET_OPTIONS_TEXT,
                  MBOX_FUNC_RESTORE|MBOX_TEXT_CENTER|MBOX_BTN_YES_NO|MBOX_DEF_BTN2, gui_menuproc_reset_selected);
}

//-------------------------------------------------------------------
//							CHDK settings menu
//-------------------------------------------------------------------

static CMenuItem chdk_settings_menu_items[] = {
    MENU_ITEM   (0,LANG_MENU_MAIN_OSD_PARAM,             MENUITEM_SUBMENU,   &osd_submenu, 0 ),
    MENU_ITEM   (0,LANG_MENU_MENU_SETTINGS,              MENUITEM_SUBMENU,   &menu_settings_submenu, 0 ),
    MENU_ITEM   (0,LANG_MENU_OSD_GRID_PARAMS,            MENUITEM_SUBMENU,   &grid_submenu, 0 ),
#if CAM_REMOTE
    MENU_ITEM   (0,LANG_MENU_REMOTE_PARAM,               MENUITEM_SUBMENU,   &remote_submenu, 0 ),
#endif
    MENU_ITEM   (0,LANG_MENU_MISC_SHOW_SPLASH,           MENUITEM_BOOL,      &conf.splash_show, 0 ),
    MENU_ITEM   (0,LANG_MENU_MISC_START_SOUND,           MENUITEM_BOOL,      &conf.start_sound, 0 ),
#if CAM_USE_ZOOM_FOR_MF
    MENU_ITEM   (0,LANG_MENU_MISC_ZOOM_FOR_MF,           MENUITEM_BOOL,      &conf.use_zoom_mf, 0 ),
#endif
#if CAM_ADJUSTABLE_ALT_BUTTON
    MENU_ITEM   (0,LANG_MENU_MISC_ALT_BUTTON,            MENUITEM_ENUM,      gui_alt_mode_button_enum, 0 ),
#endif
#if CAM_OPTIONAL_EXTRA_BUTTON
    MENU_ITEM   (0,LANG_MENU_MISC_EXTRA_BUTTON,          MENUITEM_ENUM,      gui_extra_button_enum, 0 ),
#endif
#if defined(CAM_ZOOM_ASSIST_BUTTON_CONTROL)
    MENU_ITEM   (0,LANG_MENU_MISC_ZOOM_ASSIST,           MENUITEM_BOOL,      &conf.zoom_assist_button_disable, 0 ),
#endif
    MENU_ITEM   (0,LANG_MENU_MISC_DISABLE_LCD_OFF,       MENUITEM_ENUM,      gui_alt_power_enum, 0 ),
    MENU_ITEM   (0,LANG_MENU_MAIN_RESET_OPTIONS,         MENUITEM_PROC,      gui_menuproc_reset, 0 ),
    MENU_ITEM   (0,LANG_MENU_BACK,                       MENUITEM_UP, 0, 0 ),
    {0}
};

CMenu chdk_settings_menu = {0,LANG_MENU_CHDK_SETTINGS, chdk_settings_menu_items };

//-------------------------------------------------------------------
//								Script menu
//-------------------------------------------------------------------

extern CMenu script_submenu;

static CMenuItem root_menu_items[] = {
    MENU_ITEM   (0,LANG_MENU_MAIN_SCRIPT_PARAM,          MENUITEM_SUBMENU,   &script_submenu,    0 ),
    MENU_ITEM   (0,LANG_MENU_CHDK_SETTINGS,              MENUITEM_SUBMENU,   &chdk_settings_menu, 0 ),
    MENU_ITEM   (0,LANG_MENU_MAIN_MISC,                  MENUITEM_SUBMENU,   &misc_submenu,      0 ),
    {0}
};

CMenu root_menu = {0,LANG_MENU_MAIN_TITLE, root_menu_items };

//-------------------------------------------------------------------
//					Gui On/Off and ND filter enumeration
//-------------------------------------------------------------------

const char* gui_on_off_enum(int change, int *conf_val)
{
    static const char* modes[]={ "Off", "On"};
    return gui_change_simple_enum(conf_val,change,modes,sizeof(modes)/sizeof(modes[0]));
}

#ifdef  CAM_TOUCHSCREEN_UI
const char* gui_nd_filter_state_enum(int change, int arg)
{
    return gui_change_simple_enum(&conf.nd_filter_state,change,gui_nd_filter_state_modes,sizeof(gui_nd_filter_state_modes)/sizeof(gui_nd_filter_state_modes[0]));
}

#endif

//-------------------------------------------------------------------
//								Splash screen
//-------------------------------------------------------------------
 
#define LOGO_WIDTH  149
#define LOGO_HEIGHT 119
 
static int gui_splash;
static char *logo = NULL;
static int logo_size, logo_text_width, logo_text_height;

static const char *text[] =
{
	"Special version '" HDK_VERSION " " BUILD_NUMBER "'",
	"for user Andrew Stephens"
};

static void init_splash()
{
    gui_splash = (conf.splash_show) ? SPLASH_TIME : 0;

    if (gui_splash)
    {
#if defined(VER_CHDK)
        const char *logo_name="A/CHDK/DATA/logo.dat";
#else   // CHDK-DE
        const char *logo_name="A/CHDK/DATA/logo_de.dat";
#endif
        logo = load_file(logo_name, &logo_size, 0);

        logo_text_height = sizeof(text)/sizeof(text[0]);
        logo_text_width = 0;

        int i;
        for (i=0; i<logo_text_height; ++i)
        {
            int l = strlen(text[i]);
            if (l > logo_text_width) logo_text_width = l;
        }

        logo_text_width = logo_text_width * FONT_WIDTH + 10;
        logo_text_height = logo_text_height * FONT_HEIGHT + 8;
    }
}

static void gui_draw_splash()
{
    coord x, y;
    int i;
    twoColors cl = MAKE_COLOR(COLOR_RED, COLOR_WHITE);

    const color logo_colors[8] =
    {
        COLOR_BLACK,
        COLOR_RED_DK,
        COLOR_RED,
        COLOR_GREY,
        COLOR_GREY_LT,
        COLOR_RED_LT,
        COLOR_TRANSPARENT,
        COLOR_WHITE
    };

    x = (camera_screen.width-logo_text_width)>>1; 
    y = ((camera_screen.height-logo_text_height)>>1) + 50;

    draw_rectangle(x, y, x+logo_text_width, y+logo_text_height, MAKE_COLOR(COLOR_RED, COLOR_RED), RECT_BORDER0|DRAW_FILLED|RECT_ROUND_CORNERS);
    for (i=0; i<sizeof(text)/sizeof(text[0]); ++i)
    {
        draw_string(x+((logo_text_width-strlen(text[i])*FONT_WIDTH)>>1), y+i*FONT_HEIGHT+4, text[i], cl,1);
    }
    if (logo)
    {
        int pos;
        int mx = 0;
        int my = 0;
        int offset_x = (camera_screen.width-LOGO_WIDTH)>>1;
        int offset_y = ((camera_screen.height-LOGO_HEIGHT)>>1) - 42;
        for (pos=0; pos<logo_size; pos++)
        {
            char data = logo[pos];
            color c = logo_colors[(data>>5) & 0x07];
            for (i=0; i<(data&0x1F)+1; i++)
            {
                if (c!=0x00)
                {
                    draw_pixel(offset_x+mx,offset_y+my,c);
                }
                if (mx == LOGO_WIDTH)
                {
                    mx = 0;
                    my++;
                }
                else
                {
                    mx++;
                }     
            }
        }
    }
}

static void gui_handle_splash(int force_redraw)
{
    if (gui_splash)
    {
        if (camera_info.state.gui_mode_none || camera_info.state.gui_mode_alt)
            if (force_redraw || (gui_splash == SPLASH_TIME))
                gui_draw_splash();

        if (--gui_splash == 0)
        {
            if (camera_info.state.gui_mode_none || camera_info.state.gui_mode_alt)
                gui_set_need_restore();
            if (logo)
                free(logo);
            logo = NULL;
        }
    }
}

//-------------------------------------------------------------------
//						Screen restoring and GUI initialisation
//-------------------------------------------------------------------
 
static gui_handler *gui_mode=0;	// current gui mode. pointer to gui_handler structure

static int gui_osd_need_restore = 0;    // Set when screen needs to be erase and redrawn
static int gui_mode_need_redraw = 0;    // Set if current mode needs to redraw itself

void gui_set_need_restore()
{
    gui_osd_need_restore = 1;
}

void gui_cancel_need_restore()
{
    gui_osd_need_restore = 0;
    gui_mode_need_redraw = 0;
}

void gui_set_need_redraw()
{
    gui_mode_need_redraw = 1;
}
 
void gui_init()
{
    gui_set_mode(&defaultGuiHandler);
    if (conf.start_sound > 0)
    {
        play_sound(4);
    }

    init_splash();

    draw_init();
}
 
gui_mode_t gui_get_mode() { return gui_mode->mode; }

 gui_handler* gui_set_mode(gui_handler *mode) // Set new GUI mode, returns old mode
{
    // Set up gui mode & state variables
    camera_info.state.gui_mode = mode->mode;
    camera_info.state.gui_mode_none = (camera_info.state.gui_mode == GUI_MODE_NONE);
    camera_info.state.gui_mode_alt = (camera_info.state.gui_mode == GUI_MODE_ALT);
	
	if ( gui_mode == mode )
		return gui_mode;

#ifdef CAM_TOUCHSCREEN_UI
    if (((gui_mode->mode == GUI_MODE_NONE) != (mode->mode == GUI_MODE_NONE)) || // Change from GUI_MODE_NONE to any other or vice-versa
        ((gui_mode->mode >  GUI_MODE_MENU) != (mode->mode >  GUI_MODE_MENU)))   // Switch in & out of menu mode
        redraw_buttons = 1;
#endif

    gui_handler *old_mode = gui_mode;
    gui_mode = mode;

    gui_osd_need_restore = 0;

    // Flag for screen erase/redraw unless mode is marked not to (e.g. menu box popup)
    if (((gui_mode->flags & (GUI_MODE_FLAG_NODRAWRESTORE|GUI_MODE_FLAG_NORESTORE_ON_SWITCH)) == 0) &&
        ((old_mode->flags & GUI_MODE_FLAG_NORESTORE_ON_SWITCH) == 0))
        gui_set_need_restore();
    // If old mode did not erase screen on exit then force current mode to redraw itself (e.g. exit menu popup back to file select)
    if ((old_mode->flags & (GUI_MODE_FLAG_NORESTORE_ON_SWITCH)) != 0)
        gui_set_need_redraw();
    return old_mode;
}

 void gui_chdk_draw(int force_redraw)
{
    static int clear_for_title = 1;
 gui_draw_osd();
    if (camera_info.state.osd_title_line) 
    {
        int w = camera_screen.disp_width;
        if (camera_info.state.mode_rec || camera_info.state.mode_play)
        {
            // Draw script title (up to <ALT> text, if shown)
            draw_string_clipped(camera_screen.disp_left, camera_screen.height-FONT_HEIGHT, script_title, user_color(conf.menu_color), w);
        }
        clear_for_title = 1;   
    }
    else if (clear_for_title)
    {
        clear_for_title = 0;
    }

    console_draw(force_redraw);
}

//-------------------------------------------------------------------
//			Keyboard handling for Menu and Alt when not in Menu
//-------------------------------------------------------------------
 
// Handler for Menu button press default - enter Menu mode
void gui_default_kbd_process_menu_btn()
{
    gui_set_mode(&menuGuiHandler);
}
 
 static int alt_mode_script_run()
{
    int remote_script_start_ready = 0;

    // Start the current script if script_start is enabled, we are in <ALT> mode and there is a pulse longer than 100mSec on USB port
    if (conf.remote_enable && conf.remote_enable_scripts && get_usb_power(SINGLE_PULSE) > 5)
        remote_script_start_ready=1;

    // Start a script if the shutter button pressed in <ALT> mode (kdb_blocked) or USB remote sequence not running
    //  or if script start on <ALT> enabled and this is the first pass through <ALT> mode
    if ( kbd_is_key_clicked(KEY_SHOOT_FULL) || remote_script_start_ready || script_run_on_alt_flag ) 
    {
        script_run_on_alt_flag = 0 ;
        script_start_gui(0);
        return 1;
    }

    return 0;
}

// Main button processing for CHDK Alt mode (not in MENU mode)
// This needs to be cleaned up, re-organised and commented !!!!
int gui_chdk_kbd_process()
{
    if (alt_mode_script_run()) return 0;
    if (camera_info.state.is_shutter_half_press) return 0;
   return 0;
}
 
// Handler for Menu button press in CHDK Alt mode (not in Menu mode)
// Enter main menu or user menu based on configuration
void gui_chdk_kbd_process_menu_btn()
{
        gui_menu_init(&root_menu);

    gui_default_kbd_process_menu_btn();
}
 
// GUI handler for <ALT> mode
gui_handler altGuiHandler = { GUI_MODE_ALT, gui_chdk_draw, gui_chdk_kbd_process, gui_chdk_kbd_process_menu_btn, 0, 0, };

//-------------------------------------------------------------------
//						GUI redraw functions
//-------------------------------------------------------------------
// Main GUI redraw function, perform common initialisation then calls the redraw handler for the mode
void gui_redraw()
{
    int flag_gui_enforce_redraw = 0;

    if (!draw_test_guard() && (!camera_info.state.gui_mode_none || gui_splash))     // Attempt to detect screen erase in <Alt> mode, redraw if needed
    {
        draw_set_guard();
        flag_gui_enforce_redraw = 1;
#ifdef CAM_TOUCHSCREEN_UI
        redraw_buttons = 1;
#endif
    }

#ifdef CAM_TOUCHSCREEN_UI
    extern void virtual_buttons();
    virtual_buttons();
#endif

    // Erase screen if needed
    if (gui_osd_need_restore)
    {
        draw_restore();
        gui_osd_need_restore = 0;
        flag_gui_enforce_redraw = 1;
    }

    // Force mode redraw if needed
    if (gui_mode_need_redraw)
    {
        gui_mode_need_redraw = 0;
	    flag_gui_enforce_redraw = 1;
    }
     gui_handle_splash(flag_gui_enforce_redraw);
     gui_draw_sdm_osd();
    // Call redraw handler
    if (gui_mode->redraw)
        gui_mode->redraw(flag_gui_enforce_redraw);
}

//-------------------------------------------------------------------
//               Main kbd processing for GUI modes
// Return:
//          0 = normal
//          1 = block buttons pressed from Camera firmware
//-------------------------------------------------------------------

int gui_kbd_process()
{
    if (gui_mode)
    {
        // Call menu button handler if menu button pressed
        if (gui_mode->kbd_process_menu_btn)
        {
            if (kbd_is_key_clicked(KEY_MENU))
            {
                gui_mode->kbd_process_menu_btn();
                return 0;
            }
        }

        // Call mode handler for other buttons
        if (gui_mode->kbd_process) return gui_mode->kbd_process();
    }
    return 0;
}

// Handle touch screen presses
int gui_touch_process(int x, int y)
{
    if (gui_mode && gui_mode->touch_handler)
        return gui_mode->touch_handler(x, y);
    return 0;
}

//------------------------------------------------------------------- 
static int gui_current_alt_state = ALT_MODE_NORMAL;

// Called from the KBD task code to change ALT mode state
void gui_set_alt_mode_state(int new_state)
{
    gui_current_alt_state = new_state;
}

// Called from the GUI task code to set the ALT mode state
void gui_activate_alt_mode()
{
    extern gui_handler scriptGuiHandler;

    switch (gui_current_alt_state)
    {
    case ALT_MODE_ENTER:
        
	    if (camera_info.state.state_kbd_script_run)
	        gui_set_mode(&scriptGuiHandler);
		else
	        gui_set_mode(&altGuiHandler);

        conf_update_prevent_shutdown();
        
        vid_turn_off_updates();
        break;

    case ALT_MODE_LEAVE:
        conf_save();

        // Unload all modules which are marked as safe to unload, or loaded for menus
        module_exit_alt();
        vid_turn_on_updates();
        gui_set_mode(&defaultGuiHandler);

	    conf_update_prevent_shutdown();
        break;
    }

    // Reset to stable state
    gui_current_alt_state = ALT_MODE_NORMAL;
}
