#include "camera_info.h"
#include "stdlib.h"
#include "conf.h"
#include "keyboard.h"
#include "font.h"
#include "lang.h"
#include "gui.h"
#include "gui_draw.h"
#include "gui_menu.h"
#include "gui_lang.h"

//-------------------------------------------------------------------
#define MENUSTACK_MAXDEPTH  4

//-------------------------------------------------------------------
typedef struct {
    CMenu       *menu;
    int         curpos;
    int         toppos;
} CMenuStacked;

//-------------------------------------------------------------------
extern char         selected_file[];
static CMenu        *curr_menu;
static CMenuStacked gui_menu_stack[MENUSTACK_MAXDEPTH];
static unsigned int gui_menu_stack_ptr;
static int          gui_menu_curr_item;
static int          gui_menu_top_item;
static int          gui_menu_redraw;
static int          count;  // used by touch handler
static int          x, y;
static int          w, num_lines;
static int          len_bool, len_int, len_enum, len_space, len_br1, len_br2, cl_rect;
static int          int_incr = 1;
static confColor    *item_color;
static int          item_color_type;    // MENUITEM_COLOR_FG / MENUITEM_COLOR_BG used by gui_menu_color_selected()
static OSD_pos      menu_title_pos;
static struct 
{
int x;
int yy;
int w;
}title;

static int xx,yy,imenu;
static twoColors cl;
time_t t,menu_time; 
static char mybuf[64];  
 
//-------------------------------------------------------------------
static void gui_menu_set_curr_menu(CMenu *menu_ptr, int top_item, int curr_item) {
    curr_menu = menu_ptr;
    gui_menu_top_item = top_item;
    gui_menu_curr_item = curr_item;
}

CMenu* get_curr_menu()
{
    return curr_menu;
}

int str_width(const char *str)
{
 return strlen(str)*FONT_WIDTH;
}

void gui_menu_init(CMenu *menu_ptr) 
{
   int width  = camera_screen.width;
   int height = camera_screen.height;

    if (menu_ptr) {
        if (conf.menu_select_first_entry)
            gui_menu_set_curr_menu(menu_ptr, 0, 0);
        else 
            gui_menu_set_curr_menu(menu_ptr, 0, -1);
        gui_menu_stack_ptr = 0;
    }
if((conf.camera_orientation==1)||(conf.camera_orientation==3))        // camera vertical
    {
     w = height;
     height = width;
     x = 0;
 #if defined(CAMERA_s100)  
    if(conf.camera_orientation==3)x = 30;
 #endif   
    }

   else
 
    { 
#if defined(CAMERA_g11)|| defined (CAMERA_s90)|| defined (CAMERA_s95)|| defined (CAMERA_s100)
    w = screen_width-10-10;
#else
      w = width-32-32; 
#endif   
      x = 30; 
    }
    num_lines = (camera_screen.height - camera_screen.ts_menu_border*2)/FONT_HEIGHT-1;
    y = ((height-num_lines*FONT_HEIGHT)>>1);
    len_bool = FONT_WIDTH;
    len_int = 6*FONT_WIDTH;
    len_enum = 7*FONT_WIDTH;
    len_space = FONT_WIDTH;
    len_br1 = FONT_WIDTH;
    len_br2 = FONT_WIDTH;
    cl_rect = FONT_HEIGHT - 4;

    int_incr = 1;
    
    gui_menu_redraw=2;
}
 
// -------------------------------------------------------------------
void draw_string_centre_len(int x, int y, int len, const char *str, twoColors cl,int m) 
{
 int spaces = len/FONT_WIDTH - strlen(str);
 int left = (spaces/2)+strlen(str);
  sprintf(mybuf,"%*s",left,str);
  draw_string(x,y,mybuf,cl,m);
  spaces = spaces-spaces/2;
  sprintf(mybuf,"%*s",spaces," ");         
  draw_string(x+(left*FONT_WIDTH),y,mybuf,cl,m);
}
// -------------------------------------------------------------------
int draw_string_len(int x, int y, int len, const char *str, twoColors cl,int m) 
{
  sprintf(mybuf,"%-*s",len/8,str);
  draw_string(x,y,mybuf,cl,m); 
  return len;
}
// -------------------------------------------------------------------
int draw_string_right_len(int x, int y, int len, const char *str, twoColors cl,int m) 
{
  sprintf(mybuf,"%*.*s",len/8,len/8,str);
  draw_string(x,y,mybuf,cl,m);
  return len;
} 
//-------------------------------------------------------------------
static int gui_menu_rows()
{
    int n;
    // Count the number of rows in current menu
    for(n = 0; curr_menu->menu[n].text; n++);
    return n;
}

//-------------------------------------------------------------------
// Full screen erase and redraw of menu
void gui_menu_erase_and_redraw()
{
    gui_menu_redraw = 2;
    gui_set_need_restore();
}
void gui_menu_cancel_redraw()
{
    gui_menu_redraw = 0;
    gui_cancel_need_restore();
}

//-------------------------------------------------------------------
// Function passed to gui_palette_init
// This is called when a new color is selected to update the menu / config value
static void gui_menu_color_selected(chdkColor clr)
{
    if (item_color_type == MENUITEM_COLOR_FG)
    {
        item_color->fg = clr;
    }
    else
    {
        item_color->bg = clr;
    }
    gui_menu_erase_and_redraw();
}

//-------------------------------------------------------------------
// Return to previous menu on stack
void gui_menu_back()
{
    if (gui_menu_stack_ptr > 0)
    {
        gui_menu_stack_ptr--;
        gui_menu_set_curr_menu(gui_menu_stack[gui_menu_stack_ptr].menu, gui_menu_stack[gui_menu_stack_ptr].toppos, gui_menu_stack[gui_menu_stack_ptr].curpos);
        gui_menu_erase_and_redraw();
    }
    else
    {
        // 'Back' selected; but no menu to go back to
        // Occurs when script menu opened using 'Func/Set' button
        // Return to normal <ALT> mode.
        gui_set_mode(&altGuiHandler);
    }
}
 
 //-------------------------------------------------------------------
// Helper functions for gui_menu_kbd_process
//  common code blocks extracted to try and make it easier to understand

// After updating a value check for callback and on_change functions and call if necessary
static void do_callback(const CMenuItem *mi)
{
    if ((mi->type & MENUITEM_ARG_MASK) == MENUITEM_ARG_CALLBACK && mi->arg)
    {
        ((void (*)())(mi->arg))();
    }
}

// Update an 'int' value, direction = 1 for increment, -1 for decrement
static void update_int_value(const CMenuItem *mi, int direction)
{
    // do update
    *(mi->value) += int_incr * direction;

    // Limit new value to defined bounds
    if ((mi->type & MENUITEM_F_UNSIGNED) || (mi->type & MENUITEM_SD_INT))
    {
        if (*(mi->value) < 0) 
            *(mi->value) = 0;

        if ( mi->type & MENUITEM_F_MIN)
        {
            if (*(mi->value) < MENU_MIN_UNSIGNED(mi->arg)) 
                *(mi->value) = MENU_MIN_UNSIGNED(mi->arg);
        }
    }
    else
    {
        if (*(mi->value) < -9999) 
            *(mi->value) = -9999;

        if ( mi->type & MENUITEM_F_MIN)
        {
            if (*(mi->value) < MENU_MIN_SIGNED(mi->arg)) 
                *(mi->value) = MENU_MIN_SIGNED(mi->arg);
        }
    }

    int maxval = (mi->type & MENUITEM_SD_INT) ? 9999999 : 99999;
    if (*(mi->value) > maxval)
        *(mi->value) = maxval;

    if (mi->type & MENUITEM_F_UNSIGNED)
    {
        if ( mi->type & MENUITEM_F_MAX)
        {
            if (*(mi->value) > MENU_MAX_UNSIGNED(mi->arg)) 
                *(mi->value) = MENU_MAX_UNSIGNED(mi->arg);
        }
    }
    else
    {
        if (mi->type & MENUITEM_F_MAX)
        {
            if (*(mi->value) > MENU_MAX_SIGNED(mi->arg)) 
                *(mi->value) = MENU_MAX_SIGNED(mi->arg);
        }
    }

    // execute custom callback and on_change functions
    do_callback(mi);

    // force menu redraw
    gui_menu_redraw=1;
}

// Update a 'bool' value
static void update_bool_value(const CMenuItem *mi)
{
    // update value
    *(mi->value) = !(*(mi->value));

    // execute custom callback and on_change functions
    do_callback(mi);

    // force menu redraw
    gui_menu_redraw=1;
}

// Update an 'enum' value, direction = 1 for increment, -1 for decrement
static void update_enum_value(const CMenuItem *mi, int direction)
{
    // update value
    if (mi->value)
    {
        if (mi->type & (MENUITEM_DECIMAL | MENUITEM_SD_INT | MENUITEM_HHMMSS))
            direction *= int_incr;
		/*
        if ((mi->type & MENUITEM_MASK) == MENUITEM_ENUM)
        {
            ((const char* (*)(int change, int arg))(mi->value))(direction, mi->arg);
        }
        else
        {
          */  extern const char* gui_change_enum2(const CMenuItem *menu_item, int change);
            gui_change_enum2(mi, direction);
     //   }
    }

    // force menu redraw
    gui_menu_redraw=1;
}
 
static int isText(int n)
{
    return (
            (curr_menu->menu[n].type & MENUITEM_MASK) == MENUITEM_TEXT ||
            (curr_menu->menu[n].type & MENUITEM_MASK) == MENUITEM_ERROR ||
            (curr_menu->menu[n].type & MENUITEM_MASK) == MENUITEM_WARNING ||
            (curr_menu->menu[n].type & MENUITEM_MASK) == MENUITEM_SEPARATOR
           );
}

// Open a sub-menu
void gui_activate_sub_menu(CMenu *sub_menu)
{
    // push current menu on stack
    gui_menu_stack[gui_menu_stack_ptr].menu = curr_menu;
    gui_menu_stack[gui_menu_stack_ptr].curpos = gui_menu_curr_item;
    gui_menu_stack[gui_menu_stack_ptr].toppos = gui_menu_top_item;

    // Select first item in menu, (or none)
    if (conf.menu_select_first_entry)
    {
        gui_menu_set_curr_menu(sub_menu, 0, 0);
        if (isText(gui_menu_curr_item))
        {
            //++gui_menu_top_item;
            ++gui_menu_curr_item;
        }
    }
    else 
        gui_menu_set_curr_menu(sub_menu, 0, -1);

    gui_menu_stack_ptr++;

    // FIXME check on stack overrun;
    if (gui_menu_stack_ptr > MENUSTACK_MAXDEPTH)
    {
        draw_string(1, 0, "E1", MAKE_COLOR(COLOR_RED, COLOR_YELLOW),1);
        gui_menu_stack_ptr = 0;
    }
 
    // Force full redraw
    gui_menu_erase_and_redraw();
}
 
static int gui_menu_touch_handler(int tx, int ty)
{
    int h = ((count > num_lines) ? num_lines : count) * FONT_HEIGHT;
    if ((tx >= x) && (ty >= y) && (tx < (x + w)) && (ty < (y + h)))
    {
        int r = ((ty - y) / FONT_HEIGHT) + gui_menu_top_item;
        if (!isText(r))
        {
            if (gui_menu_curr_item != r)
            {
                gui_menu_curr_item = r;
                // Redraw menu if needed
                if (gui_menu_redraw == 0) gui_menu_redraw = 1;
            }
            return KEY_SET;
        }
    }
    return 0;
}

void menu_title(x,y,w)
{
 int rec=camera_info.state.mode_rec;	
 t=time(NULL);                                                  // epoch time
 menu_time=t;                                                    
 draw_string_centre_len(x,y,w,lang_str(curr_menu->title),MAKE_COLOR(COLOR_BLUE,COLOR_WHITE),1);
}

//-------------------------------------------------------------------
//							SDM Menu Keyboard
//-------------------------------------------------------------------

int gui_menu_kbd_process() 
{
 static char sbuf[7];
 int rec=camera_info.state.mode_rec;
 int zi;
 unsigned char ac_key;
 static unsigned char prev_key;
 static unsigned int pressedDuration=0;
 if(conf.camera_orientation==1) zi=1;  // vertical rotated left
 else zi=0;
 ac_key = kbd_get_pressed_key();
 
 if(ac_key)
 {
  ++pressedDuration; 
  prev_key=ac_key;
 }
 
 else if(!ac_key && pressedDuration)
 {
  pressedDuration=0;
 switch(get_vkey(prev_key))
 {
// --------------------------------------------------------------------------------------
//                                        KEY UP
// --------------------------------------------------------------------------------------
 
        case KEY_UP:
            do 
             {
                if (gui_menu_curr_item>0) 
                {
                    if (gui_menu_curr_item-1==gui_menu_top_item && gui_menu_top_item>0) 
                        --gui_menu_top_item;
                    --gui_menu_curr_item;
                } 
               else 
                {
                    int i;
                    while (curr_menu->menu[gui_menu_curr_item+1].text)
                        ++gui_menu_curr_item;
                    gui_menu_top_item = gui_menu_curr_item - num_lines +1;
                    if (gui_menu_top_item<0) gui_menu_top_item=0;
                }
             } while ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_TEXT || 
                     (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_SEPARATOR);
#if CAM_HAS_ZOOM_LEVER
              if(gui_menu_stack_ptr)
               {
		    sprintf(sbuf, "±%d",int_incr);
		    draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"     ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
		    draw_string(conf.increment_pos.x,conf.increment_pos.y,sbuf,MAKE_COLOR(COLOR_GREY_DK_TRANS,COLOR_WHITE),1);
               }
#endif
            gui_menu_redraw=1;
            break;


// --------------------------------------------------------------------------------------
//                                        KEY DOWN
// --------------------------------------------------------------------------------------
 
        case KEY_DOWN:
            do 
             {
                if (curr_menu->menu[gui_menu_curr_item+1].text) 
                 {
                    int i;
                    for (i=0; i<num_lines-1 && curr_menu->menu[gui_menu_top_item+i].text; ++i);
                    if (i==num_lines-1 && curr_menu->menu[gui_menu_top_item+i].text 
                        && gui_menu_top_item+i-1==gui_menu_curr_item && curr_menu->menu[gui_menu_top_item+i+1].text)
                        ++gui_menu_top_item;
                    ++gui_menu_curr_item;
                 } 
                else 
                 {
                    gui_menu_curr_item = gui_menu_top_item = 0;
                 }
             } while ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_TEXT || 
                     (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_SEPARATOR);
#if CAM_HAS_ZOOM_LEVER
              if(gui_menu_stack_ptr)
               {
		    sprintf(sbuf, "±%d",int_incr);
		    draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"     ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
		    draw_string(conf.increment_pos.x,conf.increment_pos.y,sbuf,MAKE_COLOR(COLOR_GREY_DK_TRANS,COLOR_WHITE),1);
               }
#endif
            gui_menu_redraw=1;
            break;
 // --------------------------------------------------------------------------------------
//                                        KEY LEFT
// --------------------------------------------------------------------------------------     

        case KEY_LEFT:
            if (gui_menu_curr_item>=0) 
              {
                switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK) 
                  {
                    case MENUITEM_INT:
                        switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) 
                           {
                            case MENUITEM_ARG_INC:
                                *(curr_menu->menu[gui_menu_curr_item].value) -= curr_menu->menu[gui_menu_curr_item].arg;
                                break;
                            case MENUITEM_ARG_ADDR_INC:
                                *(curr_menu->menu[gui_menu_curr_item].value) -= *(int *)(curr_menu->menu[gui_menu_curr_item].arg);
                                break;
                            default:
                                *(curr_menu->menu[gui_menu_curr_item].value) -= int_incr;
                                break;

                           }
                        if (*(curr_menu->menu[gui_menu_curr_item].value) < -9999) *(curr_menu->menu[gui_menu_curr_item].value) = -9999;
                        if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_UNSIGNED) 
                           {
                            if (*(curr_menu->menu[gui_menu_curr_item].value) < 0) *(curr_menu->menu[gui_menu_curr_item].value) = 0;
                            if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_MIN) 
                              {
                                if (*(curr_menu->menu[gui_menu_curr_item].value) < (unsigned short)(curr_menu->menu[gui_menu_curr_item].arg & 0xFFFF)) 
                                    *(curr_menu->menu[gui_menu_curr_item].value) = (unsigned short)(curr_menu->menu[gui_menu_curr_item].arg & 0xFFFF);
                              }
                            } 
                            else 
                            {
                            if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_MIN) 
                              {
                                if (*(curr_menu->menu[gui_menu_curr_item].value) < (short)(curr_menu->menu[gui_menu_curr_item].arg & 0xFFFF)) 
                                    *(curr_menu->menu[gui_menu_curr_item].value) = (short)(curr_menu->menu[gui_menu_curr_item].arg & 0xFFFF);
                              }
                             }
                        if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) == MENUITEM_ARG_CALLBACK && curr_menu->menu[gui_menu_curr_item].arg) 
                           {
                            ((void (*)())(curr_menu->menu[gui_menu_curr_item].arg))();
                           }
						   /*
                        if (curr_menu->on_change) 
                           {
                            curr_menu->on_change(gui_menu_curr_item);
                           }
						   */
#if CAM_HAS_ZOOM_LEVER
                        if (incr_modified) 
                           {
				    incr_modified=0;
                            draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"    ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
				   }
#endif
                        gui_menu_redraw=1;
                        break;

                    case MENUITEM_ENUM:
                        if (curr_menu->menu[gui_menu_curr_item].value) {
                            ((const char* (*)(int change, int arg))(curr_menu->menu[gui_menu_curr_item].value))(-1, curr_menu->menu[gui_menu_curr_item].arg);
                        }
						/*
                        if (curr_menu->on_change) 
                        {
                         curr_menu->on_change(gui_menu_curr_item);
                        }
						*/
                        gui_menu_redraw=1;
                        break;
					case MENUITEM_ENUM2:
                        update_enum_value(&curr_menu->menu[gui_menu_curr_item],-1);
                        break;				
						
                    case MENUITEM_UP:
                        gui_menu_back();
                        break;
                }
            } 
			else 
			{
                gui_menu_back();
            }
            break;

// --------------------------------------------------------------------------------------
//                                        KEY RIGHT
// --------------------------------------------------------------------------------------

        case KEY_RIGHT:
            if (gui_menu_curr_item>=0) 
			{
                switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK){
                    case MENUITEM_INT:

                        switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) 
						{
                            case MENUITEM_ARG_INC:
                                *(curr_menu->menu[gui_menu_curr_item].value) += curr_menu->menu[gui_menu_curr_item].arg;
                                break;
                            case MENUITEM_ARG_ADDR_INC:
                                *(curr_menu->menu[gui_menu_curr_item].value) += *(int *)(curr_menu->menu[gui_menu_curr_item].arg);
                                break;
                            default:
                                *(curr_menu->menu[gui_menu_curr_item].value) += int_incr;
                                break;
                        }
                        if (*(curr_menu->menu[gui_menu_curr_item].value) > 999999) 
                            *(curr_menu->menu[gui_menu_curr_item].value) = 999999;
                        if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_UNSIGNED) {
                            if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_MAX) {
                                if (*(curr_menu->menu[gui_menu_curr_item].value) > (unsigned short)((curr_menu->menu[gui_menu_curr_item].arg>>16) & 0xFFFF)) 
                                    *(curr_menu->menu[gui_menu_curr_item].value) = (unsigned short)((curr_menu->menu[gui_menu_curr_item].arg>>16) & 0xFFFF);
                            }
                        } else {
                            if ( curr_menu->menu[gui_menu_curr_item].type & MENUITEM_F_MAX) {
                                if (*(curr_menu->menu[gui_menu_curr_item].value) > (short)((curr_menu->menu[gui_menu_curr_item].arg>>16) & 0xFFFF)) 
                                    *(curr_menu->menu[gui_menu_curr_item].value) = (short)((curr_menu->menu[gui_menu_curr_item].arg>>16) & 0xFFFF);
                            }
                        }
                        if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) == MENUITEM_ARG_CALLBACK && curr_menu->menu[gui_menu_curr_item].arg) {
                            ((void (*)())(curr_menu->menu[gui_menu_curr_item].arg))();
                        }
						/*
                        if (curr_menu->on_change) {
                            curr_menu->on_change(gui_menu_curr_item);
                        }
*/
#if CAM_HAS_ZOOM_LEVER
                        if (incr_modified) 
                         {
				  incr_modified=0;
                          draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"    ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
				 }
#endif

                        gui_menu_redraw=1;
                        break;


                    case MENUITEM_ENUM:
                        if (curr_menu->menu[gui_menu_curr_item].value) 
						{
                            ((const char* (*)(int change, int arg))(curr_menu->menu[gui_menu_curr_item].value))(+1, curr_menu->menu[gui_menu_curr_item].arg);
                        }
						/*
                        if (curr_menu->on_change)                     // if there is callback
                        {
                         curr_menu->on_change(gui_menu_curr_item);    // call callback function
                        }
						*/
                        gui_menu_redraw=1;
                        break;
						
					case MENUITEM_ENUM2:
                        update_enum_value(&curr_menu->menu[gui_menu_curr_item],1);
                        break;					
						
                    case MENUITEM_SUBMENU:
                        gui_menu_stack[gui_menu_stack_ptr].menu = curr_menu;
                        gui_menu_stack[gui_menu_stack_ptr].curpos = gui_menu_curr_item;
                        gui_menu_stack[gui_menu_stack_ptr].toppos = gui_menu_top_item;
                        gui_menu_set_curr_menu((CMenu*)(curr_menu->menu[gui_menu_curr_item].value), 0, 0);
                            if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_TEXT || (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_SEPARATOR) 
                            {
                              ++gui_menu_curr_item;
                            }

                        gui_menu_stack_ptr++;
                        // FIXME check on stack overrun;
                        if (gui_menu_stack_ptr > MENUSTACK_MAXDEPTH)
                         {
                            draw_txt_string(0, 0, "E1", MAKE_COLOR(COLOR_RED, COLOR_YELLOW));
                            gui_menu_stack_ptr = 0;
                         }
						gui_menu_redraw=2;
                        draw_restore();

						 break;
				}
			}
            break;

// --------------------------------------------------------------------------------------
//                                        KEY SET
// --------------------------------------------------------------------------------------
        case KEY_SET:
            if (gui_menu_curr_item>=0) 
			{
                switch (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK){

                    case MENUITEM_BOOL:
                        *(curr_menu->menu[gui_menu_curr_item].value) =
                                !(*(curr_menu->menu[gui_menu_curr_item].value));
                        if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_ARG_MASK) == MENUITEM_ARG_CALLBACK && curr_menu->menu[gui_menu_curr_item].arg) {
                            ((void (*)())(curr_menu->menu[gui_menu_curr_item].arg))();
                        }
						/*
                        if (curr_menu->on_change) {
                            curr_menu->on_change(gui_menu_curr_item);
                        }
						*/
                        gui_menu_redraw=1;
                        break;

                    case MENUITEM_PROC:
                    	if (curr_menu->menu[gui_menu_curr_item].value) {
                            ((void (*)(int arg))(curr_menu->menu[gui_menu_curr_item].value))(curr_menu->menu[gui_menu_curr_item].arg);
							/*
                            if (curr_menu->on_change) {
                                curr_menu->on_change(gui_menu_curr_item);
                            }
							*/
                            gui_menu_set_curr_menu(curr_menu, 0, -1);
                            gui_menu_redraw=2;
                        }
                        break;

                    case MENUITEM_SUBMENU:
                        gui_menu_stack[gui_menu_stack_ptr].menu = curr_menu;
                        gui_menu_stack[gui_menu_stack_ptr].curpos = gui_menu_curr_item;
                        gui_menu_stack[gui_menu_stack_ptr].toppos = gui_menu_top_item;
                        gui_menu_set_curr_menu((CMenu*)(curr_menu->menu[gui_menu_curr_item].value), 0, 0);
                            if ((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_TEXT || (curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_SEPARATOR) 
                             {
                               ++gui_menu_curr_item;
                             }
                        gui_menu_stack_ptr++;
                        // FIXME check on stack overrun;
                        if (gui_menu_stack_ptr > MENUSTACK_MAXDEPTH)
                        {
                      //      draw_txt_string(0, 0, "E1", MAKE_COLOR((rec)?COLOR_RED:COLOR_RED_P,(rec)? COLOR_YELLOW:COLOR_YELLOW_P));
					   draw_txt_string(0, 0, "E1", MAKE_COLOR( COLOR_RED , COLOR_YELLOW ));
                            gui_menu_stack_ptr = 0;
                        }

                        gui_menu_redraw=2;
                        draw_restore();
                       // gui_force_restore();
                        break;

                    case MENUITEM_UP:
                        gui_menu_back();
                        break;

                    case MENUITEM_COLOR_FG:

                    case MENUITEM_COLOR_BG:
                        draw_restore();
						/*
                        item_color=((unsigned char*)(curr_menu->menu[gui_menu_curr_item].value)) + (((curr_menu->menu[gui_menu_curr_item].type & MENUITEM_MASK)==MENUITEM_COLOR_BG)?1:0);
                        gui_palette_init(PALETTE_MODE_SELECT, (*item_color)&0xFF, gui_menu_color_selected);
                        gui_set_mode(&GUI_MODE_PALETTE_MODULE);
						*/
                        gui_menu_redraw=2;
                        break;

                    case MENUITEM_ENUM:
                        if (curr_menu->menu[gui_menu_curr_item].value) {
                            ((const char* (*)(int change, int arg))(curr_menu->menu[gui_menu_curr_item].value))(+1, curr_menu->menu[gui_menu_curr_item].arg);
                        }
                        gui_menu_redraw=1;
                        break;

                    case MENUITEM_ENUM2:
                        update_enum_value(&curr_menu->menu[gui_menu_curr_item],1);
                        gui_menu_redraw=1;
                        gui_menu_redraw=1;
                        break;	
                }
            }
            break;
// --------------------------------------------------------------------------------------
//                                        KEY ZOOM OUT
// --------------------------------------------------------------------------------------
case KEY_ZOOM_OUT:
//case KEY_ZOOM_OUT_SLOW:
#if CAM_HAS_ZOOM_LEVER
               if(!zi && (int_incr >= 10)) int_incr /= 10;        // inverted or normal orientation or rotated right
               else if (zi && (int_incr <= 10000))                 // inverted rotated left
                {
                 int_incr *= 10;
                }
		    sprintf(sbuf, "±%d",int_incr);
		    draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"     ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
		    draw_string(conf.increment_pos.x,conf.increment_pos.y,sbuf,MAKE_COLOR(COLOR_GREY_DK_TRANS,COLOR_WHITE),1);
#endif
            break;
// --------------------------------------------------------------------------------------
//                                        KEY ZOOM IN
// --------------------------------------------------------------------------------------
case KEY_ZOOM_IN:
//case KEY_ZOOM_IN_SLOW:
#if CAM_HAS_ZOOM_LEVER
                if (!zi &&(int_incr <= 10000)) int_incr *= 10;     // inverted or normal orientation or rotated right   
                else if (zi && (int_incr >= 10)) int_incr /= 10;     // inverted rotated left
		    sprintf(sbuf, "±%d",int_incr);
		    draw_string(FONT_WIDTH*2+conf.increment_pos.x,conf.increment_pos.y,"     ", MAKE_COLOR(COLOR_TRANSPARENT, COLOR_TRANSPARENT),1);
		    draw_string(conf.increment_pos.x,conf.increment_pos.y,sbuf,MAKE_COLOR(COLOR_GREY_DK_TRANS,COLOR_WHITE),1);
 #endif

            break;
    }
    }
	return 0;
}
 
void gui_menu_draw_initial() 
{
    static const char *f=" *** ";
    int l;
   int width  = camera_screen.width;
   int height = camera_screen.height;
int rec=camera_info.state.mode_rec;

  count = gui_menu_rows();
  
// ---------------------------------- small text size ----------------------
if(!curr_menu->title_char_size)                                       
 {

  if((conf.camera_orientation==1)||(conf.camera_orientation==3))
  {
   w=height;
   height = width;
   x=0;
  }

  else
 
  { 
   w = width-32-32;    
   x = 30; 
  }

    num_lines = (height-26-16)/FONT_HEIGHT-2;
    y = (camera_screen.height - ((num_lines - 1) * FONT_HEIGHT)) >> 1;   
    l = str_width(lang_str(curr_menu->title));
    yy = y-FONT_HEIGHT-FONT_HEIGHT/2;
    menu_title_pos.x=x;
    menu_title_pos.y=yy;
    menu_title(x,yy,w);
	y=yy+FONT_HEIGHT;
 } // small text size
 
 else                                                                                                                     // large text size
 {
     yy = 8;
     if((conf.camera_orientation==1)||(conf.camera_orientation==3))xx=0;
     else xx = (width-15*3*FONT_WIDTH)>>1;
     menu_title(xx,yy,width); 
	 y=yy+3*FONT_HEIGHT;
 }
}
 
//-------------------------------------------------------------------
//							SDM Menu drawing
//-------------------------------------------------------------------

void gui_menu_draw() 
{
    static char tbuf[64];
    int i, j;
   int width  = camera_screen.width;
   int height = camera_screen.height;
    const char *ch = "";
 int rec=camera_info.state.mode_rec;
      if((conf.camera_orientation==1)||(conf.camera_orientation==3))title.x=0;
     else title.x = (width-15*FONT_WIDTH*3)>>1;
   
  if((conf.camera_orientation==1)||(conf.camera_orientation==3))       // camera vertical 
    {
      w = height;
      height = width;
      num_lines = (height-26-16)/FONT_HEIGHT-2;
      y = ((height-num_lines*FONT_HEIGHT)>>1);
      x = 0;
    }

   else                                                        // camera horizontal
 
    { 
      num_lines = (height-26-16)/FONT_HEIGHT-2;
      w = width-32-32;
      x = 30;
     }
      if(!curr_menu->title_char_size) y = ((height-num_lines*FONT_HEIGHT)>>1);   // small text size
      else y = 8+FONT_HEIGHT*3;
 
      t=time(NULL); 
/*
      if(strcmp(lang_str(curr_menu->title),lang_str(LANG_MENU_BASIC_TITLE))==0)      // show clock in main menu 
     {    

    if(t>menu_time)
    {                                                         
      ttm = localtime(&t);                                            
      sprintf(buf,"%02d:%02d:%02d",ttm->tm_hour,ttm->tm_min,ttm->tm_sec);
      draw_big_string_centre_len(title.x,8,((conf.camera_orientation==1)||(conf.camera_orientation==3))?10*BIG_FONT_WIDTH:15*BIG_FONT_WIDTH,buf,MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P));      
    }
     }
     
    else if(strcmp(lang_str(curr_menu->title),lang_str(LANG_MENU_SCRIPT2_SUB_TITLE))==0)      // show clock in System scripts
    {    
     if(t>menu_time)
     {                                                         
      ttm = localtime(&t);                                             
     if((conf.camera_orientation!=1)&&(conf.camera_orientation!=3))   // not vertical
      {
      draw_string_centre_len(4*FONT_WIDTH,menu_title_pos.y,vid_get_bitmap_screen_width()-(8*FONT_WIDTH)-(8*FONT_WIDTH),lang_str(curr_menu->title),MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P),1);
      sprintf(buf,"%02d:%02d:%02d",ttm->tm_hour,ttm->tm_min,ttm->tm_sec); 
      draw_string(vid_get_bitmap_screen_width()-(8*FONT_WIDTH)-4*FONT_WIDTH, menu_title_pos.y,buf,MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P),1);       
      }
     else
      {
       draw_string_centre_len(0,menu_title_pos.y,vid_get_bitmap_screen_height()-(8*FONT_WIDTH),lang_str(curr_menu->title),MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P),1);
       sprintf(buf,"%02d:%02d:%02d",ttm->tm_hour,ttm->tm_min,ttm->tm_sec); 
       draw_string(vid_get_bitmap_screen_height()-(8*FONT_WIDTH), menu_title_pos.y,buf,MAKE_COLOR((rec)?COLOR_BLUE:COLOR_BLUE_P,(rec)?COLOR_WHITE:COLOR_WHITE_P),1);       
      }
     }
    } 
   */   
    if (gui_menu_redraw || (t>menu_time))
     {
        if (gui_menu_redraw==2)
            gui_menu_draw_initial();

        gui_menu_redraw=0;
       int font_height;
        for (imenu=gui_menu_top_item, i=0, yy=y; curr_menu->menu[imenu].text && i<num_lines; ++imenu, ++i)
        {
            cl = (gui_menu_curr_item==imenu)?MAKE_COLOR(COLOR_RED,COLOR_WHITE):MAKE_COLOR(COLOR_GREY_DK_TRANS,COLOR_WHITE);
          if(!curr_menu->title_char_size)                               // small text size
           {
               if((conf.camera_orientation==1)||(conf.camera_orientation==3)) xx = 0;
             else xx = x;
            }
           else
            {
               if((conf.camera_orientation==1)||(conf.camera_orientation==3)) xx = 0;            
              else xx = (width-15*FONT_WIDTH*3)>>1;
            }

            switch (curr_menu->menu[imenu].type & MENUITEM_MASK) 
          {		  
// --------------------------------------------------------------------------------------
//                                        BOOLEAN
// --------------------------------------------------------------------------------------
            case MENUITEM_BOOL:
               if((conf.camera_orientation!=1)&&(conf.camera_orientation!=3))                // only a leading space in horizontal orientations
                {
                xx+=draw_char(xx, yy, ' ', cl,1);
                xx+=draw_string_len(xx, yy, w-len_space-len_space-len_br1-len_bool-len_br2-len_space, lang_str(curr_menu->menu[imenu].text), cl,1);
                xx+=draw_string(xx, yy, " [", cl,1);
                xx+=draw_string_len(xx, yy, len_bool, (*(curr_menu->menu[imenu].value))?"\x95":"", cl,1);
                draw_string(xx, yy, "] ", cl,1);
               }

               else
                {
                 xx+=draw_string_len(xx, yy, w-len_space-len_br1-len_bool-len_br2, lang_str(curr_menu->menu[imenu].text), cl,1);
                 xx+=draw_string(xx, yy, " [", cl,1);
                 xx+=draw_string_len(xx, yy, len_bool, (*(curr_menu->menu[imenu].value))?"\x95":"", cl,1);
                 draw_string(xx, yy, "]", cl,1);
                }

                break;
// --------------------------------------------------------------------------------------
//                                        INT
// --------------------------------------------------------------------------------------
            case MENUITEM_INT:

                if((conf.camera_orientation!=1)&&(conf.camera_orientation!=3))                // only a leading space in horizontal orientations
               {
                xx+=draw_char(xx, yy, ' ', cl,1);
                xx+=draw_string_len(xx, yy, w-len_space-len_space-len_br1-len_int-len_br2-len_space, lang_str(curr_menu->menu[imenu].text), cl,1);
                xx+=draw_string(xx, yy, " [", cl,1);
                sprintf(tbuf, "%d", *(curr_menu->menu[imenu].value));
                xx+=draw_string_right_len(xx, yy, len_int, tbuf, cl,1);
                draw_string(xx, yy, "] ", cl,1);
                }

                else
               {           
                xx+=draw_string_len(xx, yy, w-len_space-len_br1-len_int-len_br2, lang_str(curr_menu->menu[imenu].text), cl,1);
                xx+=draw_string(xx, yy, " [", cl,1);
                sprintf(tbuf, "%d", *(curr_menu->menu[imenu].value));
                xx+=draw_string_right_len(xx, yy, len_int, tbuf, cl,1);
                draw_string(xx, yy, "]", cl,1);
                }
                break;
// --------------------------------------------------------------------------------------
//                                        UP, SUBMENU, PROC, TEXT
// --------------------------------------------------------------------------------------
            case MENUITEM_UP:
            case MENUITEM_SUBMENU:
            case MENUITEM_PROC:
            case MENUITEM_TEXT:

             if(!curr_menu->menu[imenu].char_size)              // small text size
              {
                if((conf.camera_orientation!=1)&&(conf.camera_orientation!=3))                // only a leading space in horizontal orientations
                {
#if CAM_MULTIPART
                if(curr_menu->menu[imenu].text==308)           // Swap partitions
                 {
                  xx+=draw_char(xx, yy, ' ', (cp==1)?MAKE_COLOR(COLOR_GREEN,COLOR_WHITE):MAKE_COLOR(COLOR_BLUE,COLOR_WHITE),1);
                  sprintf(buf,(cp==1)?"%s 1":"%s 2",lang_str(curr_menu->menu[imenu].text));
                  xx+=draw_string_len(xx, yy, w-len_space-len_space,buf,(cp==1)?MAKE_COLOR(COLOR_GREEN , COLOR_WHITE ):MAKE_COLOR( COLOR_BLUE , COLOR_WHITE ),1);
                  draw_char(xx, yy, ' ', (cp==1)?MAKE_COLOR( COLOR_GREEN , COLOR_WHITE ):MAKE_COLOR( COLOR_BLUE ,(rec)?COLOR_WHITE ),1);
                 }          
                else
                 {
#endif
                 if((curr_menu->menu[imenu].type & MENUITEM_MASK)==MENUITEM_UP)
				 {
				   xx+=draw_string(xx, yy, "< ", cl,1);
				   xx+=draw_string_len(xx, yy, w-len_space-len_space-FONT_WIDTH, lang_str(curr_menu->menu[imenu].text), cl,1);
				 }
				 else
				 {
                  xx+=draw_char(xx, yy, ' ', cl,1);
                  xx+=draw_string_len(xx, yy, w-len_space-len_space, lang_str(curr_menu->menu[imenu].text), cl,1);
				 }
                  draw_char(xx, yy, ' ', cl,1);
#if CAM_MULTIPART
                  }
#endif
                 }

                else                                         // vertical orientation
                {
                 xx+=draw_string_len(xx, yy, w, lang_str(curr_menu->menu[imenu].text), cl,1);
                }
               }
               else                                         // large text size
               {
                if(((conf.camera_orientation==1)||(conf.camera_orientation==3))&&(strcmp(lang_str(curr_menu->menu[imenu].text),lang_str(LANG_MENU_MAIN_TITLE))==0))
                 sprintf(tbuf,"%-15.15s","Settings");
                else  sprintf(tbuf,"%-15.15s",lang_str(curr_menu->menu[imenu].text));
                  draw_big_string(xx,yy,tbuf,cl);
               }

                break;

// --------------------------------------------------------------------------------------
//                                        SEPARATOR
// --------------------------------------------------------------------------------------
            case MENUITEM_SEPARATOR:
	               draw_char(x, yy, ' ', cl,1);

                if (lang_str(curr_menu->menu[imenu].text)[0])
                    sprintf(tbuf," %s ",lang_str(curr_menu->menu[imenu].text));
                else
                    tbuf[0] = 0;

                j = str_width(tbuf);
                xx += ((w - j) >> 1);

                if (xx > (x + len_space))
                {
                    draw_rectangle(x+len_space, yy, xx-1, yy+FONT_HEIGHT/2-1, MAKE_COLOR(BG_COLOR(cl), BG_COLOR(cl)), RECT_BORDER0|DRAW_FILLED);
                    draw_line(x+len_space, yy+FONT_HEIGHT/2, xx-1, yy+FONT_HEIGHT/2, FG_COLOR(cl));
                    draw_rectangle(x+len_space, yy+FONT_HEIGHT/2+1, xx-1, yy+FONT_HEIGHT-1, MAKE_COLOR(BG_COLOR(cl), BG_COLOR(cl)), RECT_BORDER0|DRAW_FILLED);
                }
                else
                {
                    xx = x;
                }

           //     if (j) xx += draw_clipped_string(xx, yy, tbuf, cl, 0, w);
                if(j) xx += draw_string_len(xx, yy, strlen(tbuf),tbuf, cl,1);
                if (xx < (x+w-len_space))
                {
                    draw_rectangle(xx, yy, x+w-len_space-1, yy+FONT_HEIGHT/2-1, MAKE_COLOR(BG_COLOR(cl), BG_COLOR(cl)), RECT_BORDER0|DRAW_FILLED);
                    draw_line(xx, yy+FONT_HEIGHT/2, x+w-1-len_space, yy+FONT_HEIGHT/2, FG_COLOR(cl));
                    draw_rectangle(xx, yy+FONT_HEIGHT/2+1, x+w-len_space-1, yy+FONT_HEIGHT-1, MAKE_COLOR(BG_COLOR(cl), BG_COLOR(cl)), RECT_BORDER0|DRAW_FILLED);
                }

                 draw_char(x+w-len_space, yy, ' ', cl,1);		
			/*
                if (lang_str(curr_menu->menu[imenu].text)[0]) 
                {
                    j = str_width(lang_str(curr_menu->menu[imenu].text));                  // pixel-width of text string
// ERR99 fix http://chdk.setepontos.com/index.php/topic,4647.msg46080.html#msg46080
                #if defined (CAMERA_g11)|| defined (CAMERA_s90)|| defined (CAMERA_s95)|| defined (CAMERA_s100)
		       xx+=((int)w-j-len_space*2)>>1;
                #else
                    xx+=(w-j-len_space*2)>>1;                                                  // starting position to centre the string
                #endif
                      draw_char(x, yy, ' ', cl,1);
                      draw_filled_rect(x+len_space, yy, xx-1, yy+FONT_HEIGHT/2-1, MAKE_COLOR(cl>>8, cl>>8));
                //      draw_line(x+len_space, yy+FONT_HEIGHT/2, xx-1, yy+FONT_HEIGHT/2, cl);
                      draw_filled_rect(x+len_space, yy+FONT_HEIGHT/2+1, xx-1, yy+FONT_HEIGHT-1, MAKE_COLOR(cl>>8, cl>>8));
                      xx+=draw_char(xx, yy, ' ', cl,1);
                      xx+=draw_string(xx, yy, lang_str(curr_menu->menu[imenu].text), cl,1);
                      xx+=draw_char(xx, yy, ' ', cl,1);            
                      draw_filled_rect(xx, yy, x+w-len_space-1, yy+FONT_HEIGHT/2-1, MAKE_COLOR(cl>>8, cl>>8));
                      draw_line(xx, yy+FONT_HEIGHT/2, x+w-1-len_space, yy+FONT_HEIGHT/2, cl);
                      draw_filled_rect(xx, yy+FONT_HEIGHT/2+1, x+w-len_space-1, yy+FONT_HEIGHT-1, MAKE_COLOR(cl>>8, cl>>8));
                      draw_char(x+w-len_space, yy, ' ', cl,1);
                }
                    draw_char(x, yy, ' ', cl,1);
                    draw_filled_rect(x+len_space, yy, x+w-len_space-1, yy+FONT_HEIGHT/2-1, MAKE_COLOR(cl>>8, cl>>8));
                    draw_line(x+len_space, yy+FONT_HEIGHT/2, x+w-1-len_space, yy+FONT_HEIGHT/2, cl);
                    draw_filled_rect(x+len_space, yy+FONT_HEIGHT/2+1, x+w-len_space-1, yy+FONT_HEIGHT-1, MAKE_COLOR(cl>>8, cl>>8));
                    draw_char(x+w-len_space, yy, ' ', cl,1);
					*/
                break;

// --------------------------------------------------------------------------------------
//                                        COLOR_FG, COLOR_BG
// --------------------------------------------------------------------------------------
            case MENUITEM_COLOR_FG:
            case MENUITEM_COLOR_BG:
                if((conf.camera_orientation!=1)&&(conf.camera_orientation!=3))                // only a leading space in horizontal orientations
                 {
                  xx+=draw_char(xx, yy, ' ', cl,1);
                  xx+=draw_string_len(xx, yy, w-len_space, lang_str(curr_menu->menu[imenu].text), cl,1);
                  draw_filled_rect(x+w-1-cl_rect-2-len_space, yy+2, x+w-1-2-len_space, yy+FONT_HEIGHT-1-2, 
                                 MAKE_COLOR(((*(curr_menu->menu[imenu].value))>>(((curr_menu->menu[imenu].type & MENUITEM_MASK)==MENUITEM_COLOR_BG)?8:0))&0xFF, (cl>>8)&0xFF));
                 }

                  else                                           // vertical
                   {
                    xx+=draw_string_len(xx, yy, w, lang_str(curr_menu->menu[imenu].text), cl,1);
                    draw_filled_rect(x+w-1-cl_rect-2, yy+2, x+w-1-2, yy+FONT_HEIGHT-1-2, 
                                 MAKE_COLOR(((*(curr_menu->menu[imenu].value))>>(((curr_menu->menu[imenu].type & MENUITEM_MASK)==MENUITEM_COLOR_BG)?8:0))&0xFF, (cl>>8)&0xFF));
                    }
                break;
// --------------------------------------------------------------------------------------
//                                                   ENUM
// --------------------------------------------------------------------------------------
            case MENUITEM_ENUM:
                if (curr_menu->menu[imenu].value) {
                    ch=((const char* (*)(int change, int arg))(curr_menu->menu[imenu].value))(0, curr_menu->menu[imenu].arg);
                }
               if((conf.camera_orientation!=1)&&(conf.camera_orientation!=3))                // not vertical
               {
                xx+=draw_char(xx, yy, ' ', cl,1);
                xx+=draw_string_len(xx, yy, w-len_space-len_space-len_br1-len_enum-len_br2-len_space, lang_str(curr_menu->menu[imenu].text), cl,1);          
                xx+=draw_string(xx, yy, " [", cl,1);
                xx+=draw_string_right_len(xx, yy, len_enum, ch, cl,1);
                draw_string(xx, yy, "] ", cl,1);
                }
                else                                             // camera vertical
                {
                 xx+=draw_string_len(xx, yy, w-len_br1-str_width(ch)-len_br2, lang_str(curr_menu->menu[imenu].text),cl,1);
                 xx+=draw_string(xx, yy, "[", cl,1);
                 xx+=draw_string(xx, yy, ch, cl,1);
                 draw_string(xx, yy, "]", cl,1);
                }

                break;
				
	        case MENUITEM_ENUM2:
                if (curr_menu->menu[imenu].value)
                {
                    extern const char* gui_change_enum2(const CMenuItem *menu_item, int change);
                    ch = gui_change_enum2(&curr_menu->menu[imenu], 0);
                }
               if((conf.camera_orientation!=1)&&(conf.camera_orientation!=3))                // not vertical
               {
                xx+=draw_char(xx, yy, ' ', cl,1);
                xx+=draw_string_len(xx, yy, w-len_space-len_space-len_br1-len_enum-len_br2-len_space, lang_str(curr_menu->menu[imenu].text), cl,1);          
                xx+=draw_string(xx, yy, " [", cl,1);
                xx+=draw_string_right_len(xx, yy, len_enum, ch, cl,1);
                draw_string(xx, yy, "] ", cl,1);
                }
                else                                             // camera vertical
                {
                 xx+=draw_string_len(xx, yy, w-len_br1-str_width(ch)-len_br2, lang_str(curr_menu->menu[imenu].text),cl,1);
                 xx+=draw_string(xx, yy, "[", cl,1);
                 xx+=draw_string(xx, yy, ch, cl,1);
                 draw_string(xx, yy, "]", cl,1);
                }	   
                break;			
				
            } // _switch

                      if(!curr_menu->menu[imenu].char_size) font_height = FONT_HEIGHT;  // small text size
                       else font_height = FONT_HEIGHT*3;
                       yy+=font_height;
        }   // _for
		
        //scrollbar
      if((count>num_lines) && (conf.camera_orientation != 1))
         {
            i=num_lines*FONT_HEIGHT-1 -1;           //full height
            j=i*num_lines/count;                    //bar height
            if (j<20) j=20;
            i=(i-j)*((gui_menu_curr_item<0)?0:gui_menu_curr_item)/(count-1);   //top pos
            draw_filled_rect((x+w)+2, y+1, 
                             (x+w)+6, y+1+i,MAKE_COLOR(COLOR_BLACK,COLOR_BLACK));
            draw_filled_rect((x+w)+2, y+i+j, 
                             (x+w)+6, y+num_lines*FONT_HEIGHT-1-1, MAKE_COLOR(COLOR_BLACK,COLOR_BLACK));
            draw_filled_rect((x+w)+2, y+1+i, 
                             (x+w)+6, y+i+j,MAKE_COLOR(COLOR_WHITE ,COLOR_WHITE));
          }

    } 
   menu_time=t;
}
 
//-------------------------------------------------------------------
// Menu button handler for Menu mode
void gui_menu_kbd_process_menu_btn()
{
 conf_save();
 gui_set_mode(&altGuiHandler);
}

//-------------------------------------------------------------------
// GUI handler for menus
gui_handler menuGuiHandler = { GUI_MODE_MENU, gui_menu_draw, gui_menu_kbd_process, gui_menu_kbd_process_menu_btn, gui_menu_touch_handler, 0 };
//-------------------------------------------------------------------
