#define PARAM_FILE_COUNTER      0x2F //
#define PARAM_EXPOSURE_COUNTER  0x01

#include "platform.h"

const ApertureSize aperture_sizes_table[] = {
    {  9, 292, "2.8" },
    { 10, 312, "3.2" },
    { 11, 334, "3.5" },
    { 12, 359, "3.5" },
    { 13, 388, "4.0" },
    { 14, 406, "4.5 *" },
    { 15, 421, "4.5" },
    { 16, 426, "5.0 *" },
    { 17, 448, "5.0 *" },
    { 18, 457, "5.1" },
    { 19, 473, "5.6 *" },
    { 20, 502, "6.3 *" },
    { 21, 535, "7.1 *" },
    { 22, 571, "8.0 *" },
}; // * denotes values with the small circular aperture swung in

const ShutterSpeed shutter_speeds_table[] = {
    { -12, -384, "15", 15000000 },
    { -11, -352, "13", 13000000 },
    { -10, -320, "10", 10000000 },
    {  -9, -288, "8",   8000000 },
    {  -8, -256, "6",   6000000 },
    {  -7, -224, "5",   5000000 },
    {  -6, -192, "4",   4000000 },
    {  -5, -160, "3.2", 3200000 },
    {  -4, -128, "2.5", 2500000 },
    {  -3,  -96, "2",   2000000 },
    {  -2,  -64, "1.6", 1600000 },
    {  -1,  -32, "1.3", 1300000 },
    {   0,    0, "1",   1000000 },
    {   1,   32, "0.8",  800000 },
    {   2,   64, "0.6",  600000 },
    {   3,   96, "0.5",  500000 },
    {   4,  128, "0.4",  400000 },
    {   5,  160, "0.3",  300000 },
    {   6,  192, "1/4",  250000 },
    {   7,  224, "1/5",  200000 },
    {   8,  256, "1/6",  166667 },
    {   9,  288, "1/8",  125000 },
    {  10,  320, "1/10", 100000 },
    {  11,  352, "1/13",  76923 },
    {  12,  384, "1/15",  66667 },
    {  13,  416, "1/20",  50000 },
    {  14,  448, "1/25",  40000 },
    {  15,  480, "1/30",  33333 },
    {  16,  512, "1/40",  25000 },
    {  17,  544, "1/50",  20000 },
    {  18,  576, "1/60",  16667 },
    {  19,  608, "1/80",  12500 },
    {  20,  640, "1/100", 10000 },
    {  21,  672, "1/125",  8000 },
    {  22,  704, "1/160",  6250 },
    {  23,  736, "1/200",  5000 },
    {  24,  768, "1/250",  4000 },
    {  25,  800, "1/320",  3125 },
    {  26,  832, "1/400",  2500 },
    {  27,  864, "1/500",  2000 },
    {  28,  896, "1/640",  1563 },
    {  29,  928, "1/800",  1250 },
    {  30,  960, "1/1000", 1000 },
    {  31,  992, "1/1250",  800 },
    {  32, 1021, "1/1600",  625 },
    {  33, 1053, "1/2000",  500 },
};

const ISOTable iso_table[] = {
    {  0,    0, "Auto", -1},
    {  1,   50,   "50", -1},
    {  2,  100,  "100", -1},
    {  3,  200,  "200", -1},
};

/*
canon mode list ffd39b10 in a410 100f
*/          
const CapturemodeMap modemap[] = {
    { MODE_P,                   0 },
    { MODE_AUTO,                4 },
    { MODE_PORTRAIT,            5 },
    { MODE_STITCH,              3 },
    { MODE_NIGHT_SNAPSHOT,     10 }, // "night snapshot"
    { MODE_KIDS_PETS,          11 }, // "kids and pets"
    { MODE_INDOOR,              9 }, // "indoor"
    { MODE_FOLIAGE,             6 }, // "foliage"
    { MODE_SNOW,                7 }, // "snow"
    { MODE_BEACH,               8 }, // "beach"
    { MODE_FIREWORK,           12 }, // "fireworks"
    { MODE_VIDEO_STD,          13 }, // there's only one mode for video (according to the modemap, this is supposed to be 0x1000d)
    { MODE_SUPER_MACRO,         1 },
    { MODE_MY_COLORS,           2 }
    /*the modemap also contains the following:
    0x1000d (not 0xd, but setting mode to 0xd (13) works), propcase 0 also reports 13
    0x1000e
    0x1000f
    trying to set 14 or 15 fails (camera crashes)
    */
};
#include "../generic/shooting.c"

/*long get_file_next_counter() {
    return get_file_counter();
}*/

long get_file_next_counter() { //looks like this hack is needed for old vxworks
    return ((get_file_counter()>>4)+1)<<4;
}

long get_target_file_num() {
    long n;
    
    n = get_file_next_counter();
    n = (n>>4)&0x3FFF;
    return n;
}

long get_target_dir_num() {
    long n;
    
    n = get_file_next_counter();
    n = (n>>18)&0x3FF;
    return n;
}
